@@ -7,7 +7,7 @@ __docformat__ = 'restructuredtext en'
 lxml based OPF parser.
 '''
 
-import re, sys, unittest, functools, os, mimetypes, uuid, glob, cStringIO, json
+import re, sys, unittest, functools, os, uuid, glob, cStringIO, json
 from urllib import unquote
 from urlparse import urlparse
 
@@ -20,7 +20,8 @@ from calibre.ebooks.metadata import string_to_authors, MetaInformation, check_is
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.utils.date import parse_date, isoformat
 from calibre.utils.localization import get_lang
-from calibre import prints
+from calibre import prints, guess_type
+from calibre.utils.cleantext import clean_ascii_chars
 
 class Resource(object): # {{{
     '''
@@ -41,7 +42,7 @@ class Resource(object): # {{{
         self.path = None
         self.fragment = ''
         try:
-            self.mime_type = mimetypes.guess_type(href_or_path)[0]
+            self.mime_type = guess_type(href_or_path)[0]
         except:
             self.mime_type = None
         if self.mime_type is None:
@@ -965,7 +966,9 @@ class OPF(object): # {{{
             cover_id = covers[0].get('content')
             for item in self.itermanifest():
                 if item.get('id', None) == cover_id:
-                    return item.get('href', None)
+                    mt = item.get('media-type', '')
+                    if 'xml' not in mt:
+                        return item.get('href', None)
 
     @dynamic_property
     def cover(self):
@@ -999,7 +1002,7 @@ class OPF(object): # {{{
                 for t in ('cover', 'other.ms-coverimage-standard', 'other.ms-coverimage'):
                     for item in self.guide:
                         if item.type.lower() == t:
-                            self.create_manifest_item(item.href(), mimetypes.guess_type(path)[0])
+                            self.create_manifest_item(item.href(), guess_type(path)[0])
 
         return property(fget=fget, fset=fset)
 
@@ -1157,7 +1160,7 @@ class OPFCreator(Metadata):
 
         def DC_ELEM(tag, text, dc_attrs={}, opf_attrs={}):
             if text:
-                elem = getattr(DC, tag)(text, **dc_attrs)
+                elem = getattr(DC, tag)(clean_ascii_chars(text), **dc_attrs)
             else:
                 elem = getattr(DC, tag)(**dc_attrs)
             for k, v in opf_attrs.items():
@@ -1260,7 +1263,6 @@ def metadata_to_opf(mi, as_string=True):
     from lxml import etree
     import textwrap
     from calibre.ebooks.oeb.base import OPF, DC
-    from calibre.utils.cleantext import clean_ascii_chars
 
     if not mi.application_id:
         mi.application_id = str(uuid.uuid4())
