@@ -10,42 +10,130 @@ __docformat__ = 'restructuredtext en'
 import socket, time, re
 from urllib import urlencode
 from threading import Thread
+from Queue import Queue, Empty
 
 from lxml.html import soupparser, tostring
 
 from calibre import as_unicode
 from calibre.ebooks.metadata import check_isbn
-from calibre.ebooks.metadata.sources.base import Source
+from calibre.ebooks.metadata.sources.base import Source, Option
 from calibre.utils.cleantext import clean_ascii_chars
 from calibre.ebooks.chardet import xml_to_unicode
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.library.comments import sanitize_comments_html
 from calibre.utils.date import parse_date
 
-class Worker(Thread): # {{{
+class Worker(Thread): # Get details {{{
 
     '''
     Get book details from amazons book page in a separate thread
     '''
 
-    def __init__(self, url, result_queue, browser, log, timeout=20):
+    def __init__(self, url, result_queue, browser, log, relevance, plugin, timeout=20):
         Thread.__init__(self)
         self.daemon = True
         self.url, self.result_queue = url, result_queue
         self.log, self.timeout = log, timeout
+        self.relevance, self.plugin = relevance, plugin
         self.browser = browser.clone_browser()
         self.cover_url = self.amazon_id = self.isbn = None
+        self.domain = self.plugin.domain
+
+        months = {
+                'de': {
+            1 : ['jän'],
+            3 : ['märz'],
+            5 : ['mai'],
+            6 : ['juni'],
+            7 : ['juli'],
+            10: ['okt'],
+            12: ['dez']
+            },
+                'it': {
+            1: ['enn'],
+            2: ['febbr'],
+            5: ['magg'],
+            6: ['giugno'],
+            7: ['luglio'],
+            8: ['ag'],
+            9: ['sett'],
+            10: ['ott'],
+            12: ['dic'],
+            },
+                'fr': {
+            1: ['janv'],
+            2: ['févr'],
+            3: ['mars'],
+            4: ['avril'],
+            5: ['mai'],
+            6: ['juin'],
+            7: ['juil'],
+            8: ['août'],
+            9: ['sept'],
+            12: ['déc'],
+            },
+
+        }
+
+        self.english_months = [None, 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+                'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
+        self.months = months.get(self.domain, {})
+
+        self.pd_xpath = '''
+            //h2[text()="Product Details" or \
+                 text()="Produktinformation" or \
+                 text()="Dettagli prodotto" or \
+                 text()="Product details" or \
+                 text()="Détails sur le produit"]/../div[@class="content"]
+            '''
+        self.publisher_xpath = '''
+            descendant::*[starts-with(text(), "Publisher:") or \
+                    starts-with(text(), "Verlag:") or \
+                    starts-with(text(), "Editore:") or \
+                    starts-with(text(), "Editeur")]
+            '''
+        self.language_xpath =    '''
+            descendant::*[
+                starts-with(text(), "Language:") \
+                or text() = "Language" \
+                or text() = "Sprache:" \
+                or text() = "Lingua:" \
+                or starts-with(text(), "Langue") \
+                ]
+            '''
+        self.ratings_pat = re.compile(
+            r'([0-9.]+) (out of|von|su|étoiles sur) (\d+)( (stars|Sternen|stelle)){0,1}')
+
+        lm = {
+                'en': ('English', 'Englisch'),
+                'fr': ('French', 'Français'),
+                'it': ('Italian', 'Italiano'),
+                'de': ('German', 'Deutsch'),
+                }
+        self.lang_map = {}
+        for code, names in lm.iteritems():
+            for name in names:
+                self.lang_map[name] = code
+
+    def delocalize_datestr(self, raw):
+        if not self.months:
+            return raw
+        ans = raw.lower()
+        for i, vals in self.months.iteritems():
+            for x in vals:
+                ans = ans.replace(x, self.english_months[i])
+        return ans
 
     def run(self):
         try:
             self.get_details()
         except:
-            self.log.error('get_details failed for url: %r'%self.url)
+            self.log.exception('get_details failed for url: %r'%self.url)
 
     def get_details(self):
         try:
             raw = self.browser.open_novisit(self.url, timeout=self.timeout).read().strip()
-        except Exception, e:
+        except Exception as e:
             if callable(getattr(e, 'getcode', None)) and \
                     e.getcode() == 404:
                 self.log.error('URL malformed: %r'%self.url)
@@ -62,7 +150,7 @@ class Worker(Thread): # {{{
 
         raw = xml_to_unicode(raw, strip_encoding_pats=True,
                 resolve_entities=True)[0]
-        # open('/t/t.html', 'wb').write(raw)
+        #open('/t/t.html', 'wb').write(raw)
 
         if '<title>404 - ' in raw:
             self.log.error('URL malformed: %r'%self.url)
@@ -130,7 +218,7 @@ class Worker(Thread): # {{{
             self.log.exception('Error parsing cover for url: %r'%self.url)
         mi.has_cover = bool(self.cover_url)
 
-        pd = root.xpath('//h2[text()="Product Details"]/../div[@class="content"]')
+        pd = root.xpath(self.pd_xpath)
         if pd:
             pd = pd[0]
 
@@ -161,6 +249,17 @@ class Worker(Thread): # {{{
         else:
             self.log.warning('Failed to find product description for url: %r'%self.url)
 
+        mi.source_relevance = self.relevance
+
+        if self.amazon_id:
+            if self.isbn:
+                self.plugin.cache_isbn_to_identifier(self.isbn, self.amazon_id)
+            if self.cover_url:
+                self.plugin.cache_identifier_to_cover_url(self.amazon_id,
+                        self.cover_url)
+
+        self.plugin.clean_downloaded_metadata(mi)
+
         self.result_queue.put(mi)
 
     def parse_asin(self, root):
@@ -181,30 +280,45 @@ class Worker(Thread): # {{{
     def parse_authors(self, root):
         x = '//h1[@class="parseasinTitle"]/following-sibling::span/*[(name()="a" and @href) or (name()="span" and @class="contributorNameTrigger")]'
         aname = root.xpath(x)
+        if not aname:
+            aname = root.xpath('''
+            //h1[@class="parseasinTitle"]/following-sibling::*[(name()="a" and @href) or (name()="span" and @class="contributorNameTrigger")]
+                    ''')
         for x in aname:
             x.tail = ''
         authors = [tostring(x, encoding=unicode, method='text').strip() for x
                 in aname]
+        authors = [a for a in authors if a]
         return authors
 
     def parse_rating(self, root):
         ratings = root.xpath('//div[@class="jumpBar"]/descendant::span[@class="asinReviewsSummary"]')
-        pat = re.compile(r'([0-9.]+) out of (\d+) stars')
+        if not ratings:
+            ratings = root.xpath('//div[@class="buying"]/descendant::span[@class="asinReviewsSummary"]')
+        if not ratings:
+            ratings = root.xpath('//span[@class="crAvgStars"]/descendant::span[@class="asinReviewsSummary"]')
         if ratings:
             for elem in ratings[0].xpath('descendant::*[@title]'):
                 t = elem.get('title').strip()
-                m = pat.match(t)
+                m = self.ratings_pat.match(t)
                 if m is not None:
-                    return float(m.group(1))/float(m.group(2)) * 5
+                    return float(m.group(1))/float(m.group(3)) * 5
 
     def parse_comments(self, root):
         desc = root.xpath('//div[@id="productDescription"]/*[@class="content"]')
         if desc:
             desc = desc[0]
             for c in desc.xpath('descendant::*[@class="seeAll" or'
-                    ' @class="emptyClear" or @href]'):
+                    ' @class="emptyClear"]'):
                 c.getparent().remove(c)
+            for a in desc.xpath('descendant::a[@href]'):
+                del a.attrib['href']
+                a.tag = 'span'
             desc = tostring(desc, method='html', encoding=unicode).strip()
+
+            # Encoding bug in Amazon data U+fffd (replacement char)
+            # in some examples it is present in place of '
+            desc = desc.replace('\ufffd', "'")
             # remove all attributes from tags
             desc = re.sub(r'<([a-zA-Z0-9]+)\s[^>]+>', r'<\1>', desc)
             # Collapse whitespace
@@ -230,54 +344,84 @@ class Worker(Thread): # {{{
                         return ('/'.join(parts[:-1]))+'/'+bn
 
     def parse_isbn(self, pd):
-        for x in reversed(pd.xpath(
-            'descendant::*[starts-with(text(), "ISBN")]')):
+        items = pd.xpath(
+            'descendant::*[starts-with(text(), "ISBN")]')
+        if not items:
+            items = pd.xpath(
+                'descendant::b[contains(text(), "ISBN:")]')
+        for x in reversed(items):
             if x.tail:
                 ans = check_isbn(x.tail.strip())
                 if ans:
                     return ans
 
     def parse_publisher(self, pd):
-        for x in reversed(pd.xpath(
-            'descendant::*[starts-with(text(), "Publisher:")]')):
+        for x in reversed(pd.xpath(self.publisher_xpath)):
             if x.tail:
                 ans = x.tail.partition(';')[0]
                 return ans.partition('(')[0].strip()
 
     def parse_pubdate(self, pd):
-        for x in reversed(pd.xpath(
-            'descendant::*[starts-with(text(), "Publisher:")]')):
+        for x in reversed(pd.xpath(self.publisher_xpath)):
             if x.tail:
                 ans = x.tail
                 date = ans.partition('(')[-1].replace(')', '').strip()
+                date = self.delocalize_datestr(date)
                 return parse_date(date, assume_utc=True)
 
     def parse_language(self, pd):
-        for x in reversed(pd.xpath(
-            'descendant::*[starts-with(text(), "Language:")]')):
+        for x in reversed(pd.xpath(self.language_xpath)):
             if x.tail:
                 ans = x.tail.strip()
-                if ans == 'English':
-                    return 'en'
+                ans = self.lang_map.get(ans, None)
+                if ans:
+                    return ans
 # }}}
 
 class Amazon(Source):
 
-    name = 'Amazon'
-    description = _('Downloads metadata from Amazon')
+    name = 'Amazon.com'
+    description = _('Downloads metadata and covers from Amazon')
 
-    capabilities = frozenset(['identify'])
+    capabilities = frozenset(['identify', 'cover'])
     touched_fields = frozenset(['title', 'authors', 'identifier:amazon',
-        'identifier:isbn', 'rating', 'comments', 'publisher', 'pubdate'])
+        'identifier:isbn', 'rating', 'comments', 'publisher', 'pubdate',
+        'language'])
+    has_html_comments = True
+    supports_gzip_transfer_encoding = True
 
     AMAZON_DOMAINS = {
             'com': _('US'),
             'fr' : _('France'),
             'de' : _('Germany'),
+            'uk' : _('UK'),
+            'it' : _('Italy'),
     }
 
+    options = (
+            Option('domain', 'choices', 'com', _('Amazon website to use:'),
+                _('Metadata from Amazon will be fetched using this '
+                    'country\'s Amazon website.'), choices=AMAZON_DOMAINS),
+            )
+
+    def get_book_url(self, identifiers): # {{{
+        asin = identifiers.get('amazon', None)
+        if asin is None:
+            asin = identifiers.get('asin', None)
+        if asin:
+            return ('amazon', asin, 'http://amzn.com/%s'%asin)
+    # }}}
+
+    @property
+    def domain(self):
+        domain = self.prefs['domain']
+        if domain not in self.AMAZON_DOMAINS:
+            domain = 'com'
+
+        return domain
+
     def create_query(self, log, title=None, authors=None, identifiers={}): # {{{
-        domain = self.prefs.get('domain', 'com')
+        domain = self.domain
 
         # See the amazon detailed search page to get all options
         q = {   'search-alias' : 'aps',
@@ -310,17 +454,35 @@ class Amazon(Source):
                     q['field-author'] = ' '.join(author_tokens)
 
         if not ('field-keywords' in q or 'field-isbn' in q or
-                ('field-title' in q and 'field-author' in q)):
+                ('field-title' in q)):
             # Insufficient metadata to make an identify query
             return None
 
-        utf8q = dict([(x.encode('utf-8'), y.encode('utf-8')) for x, y in
+        latin1q = dict([(x.encode('latin1', 'ignore'), y.encode('latin1',
+            'ignore')) for x, y in
             q.iteritems()])
-        url = 'http://www.amazon.%s/s/?'%domain + urlencode(utf8q)
+        if domain == 'uk':
+            domain = 'co.uk'
+        url = 'http://www.amazon.%s/s/?'%domain + urlencode(latin1q)
         return url
 
     # }}}
 
+    def get_cached_cover_url(self, identifiers): # {{{
+        url = None
+        asin = identifiers.get('amazon', None)
+        if asin is None:
+            asin = identifiers.get('asin', None)
+        if asin is None:
+            isbn = identifiers.get('isbn', None)
+            if isbn is not None:
+                asin = self.cached_isbn_to_identifier(isbn)
+        if asin is not None:
+            url = self.cached_identifier_to_cover_url(asin)
+
+        return url
+    # }}}
+
     def identify(self, log, result_queue, abort, title=None, authors=None, # {{{
             identifiers={}, timeout=30):
         '''
@@ -335,7 +497,7 @@ class Amazon(Source):
         br = self.browser
         try:
             raw = br.open_novisit(query, timeout=timeout).read().strip()
-        except Exception, e:
+        except Exception as e:
             if callable(getattr(e, 'getcode', None)) and \
                     e.getcode() == 404:
                 log.error('Query malformed: %r'%query)
@@ -379,6 +541,19 @@ class Amazon(Source):
                     if 'bulk pack' not in title:
                         matches.append(a.get('href'))
                     break
+            if not matches:
+                # This can happen for some user agents that Amazon thinks are
+                # mobile/less capable
+                log('Trying alternate results page markup')
+                for td in root.xpath(
+                    r'//div[@id="Results"]/descendant::td[starts-with(@id, "search:Td:")]'):
+                    for a in td.xpath(r'descendant::td[@class="dataColumn"]/descendant::a[@href]/span[@class="srTitle"]/..'):
+                        title = tostring(a, method='text', encoding=unicode).lower()
+                        if ('bulk pack' not in title and '[audiobook]' not in
+                                title and '[audio cd]' not in title):
+                            matches.append(a.get('href'))
+                        break
+
 
         # Keep only the top 5 matches as the matches are sorted by relevance by
         # Amazon so lower matches are not likely to be very relevant
@@ -396,7 +571,8 @@ class Amazon(Source):
             log.error('No matches found with query: %r'%query)
             return
 
-        workers = [Worker(url, result_queue, br, log) for url in matches]
+        workers = [Worker(url, result_queue, br, log, i, self) for i, url in
+                enumerate(matches)]
 
         for w in workers:
             w.start()
@@ -414,28 +590,64 @@ class Amazon(Source):
             if not a_worker_is_alive:
                 break
 
-        for w in workers:
-            if w.amazon_id:
-                if w.isbn:
-                    self.cache_isbn_to_identifier(w.isbn, w.amazon_id)
-                if w.cover_url:
-                    self.cache_identifier_to_cover_url(w.amazon_id,
-                            w.cover_url)
-
         return None
     # }}}
 
+    def download_cover(self, log, result_queue, abort, # {{{
+            title=None, authors=None, identifiers={}, timeout=30):
+        cached_url = self.get_cached_cover_url(identifiers)
+        if cached_url is None:
+            log.info('No cached cover found, running identify')
+            rq = Queue()
+            self.identify(log, rq, abort, title=title, authors=authors,
+                    identifiers=identifiers)
+            if abort.is_set():
+                return
+            results = []
+            while True:
+                try:
+                    results.append(rq.get_nowait())
+                except Empty:
+                    break
+            results.sort(key=self.identify_results_keygen(
+                title=title, authors=authors, identifiers=identifiers))
+            for mi in results:
+                cached_url = self.get_cached_cover_url(mi.identifiers)
+                if cached_url is not None:
+                    break
+        if cached_url is None:
+            log.info('No cover found')
+            return
 
-if __name__ == '__main__':
+        if abort.is_set():
+            return
+        br = self.browser
+        log('Downloading cover from:', cached_url)
+        try:
+            cdata = br.open_novisit(cached_url, timeout=timeout).read()
+            result_queue.put((self, cdata))
+        except:
+            log.exception('Failed to download cover from:', cached_url)
+    # }}}
+
+if __name__ == '__main__': # tests {{{
     # To run these test use: calibre-debug -e
     # src/calibre/ebooks/metadata/sources/amazon.py
     from calibre.ebooks.metadata.sources.test import (test_identify_plugin,
             title_test, authors_test)
-    test_identify_plugin(Amazon.name,
-        [
+    com_tests = [ # {{{
 
-            ( # An e-book ISBN not on Amazon, one of the authors is
-              # unknown to Amazon, so no popup wrapper
+            (  # Description has links
+                {'identifiers':{'isbn': '9780671578275'}},
+                [title_test('A Civil Campaign: A Comedy of Biology and Manners',
+                    exact=True), authors_test(['Lois McMaster Bujold'])
+                 ]
+
+            ),
+
+            ( # An e-book ISBN not on Amazon, the title/author search matches
+              # the Kindle edition, which has different markup for ratings and
+              # isbn
                 {'identifiers':{'isbn': '9780307459671'},
                     'title':'Invisible Gorilla', 'authors':['Christopher Chabris']},
                 [title_test('The Invisible Gorilla: And Other Ways Our Intuitions Deceive Us',
@@ -446,7 +658,7 @@ if __name__ == '__main__':
             (  # This isbn not on amazon
                 {'identifiers':{'isbn': '8324616489'}, 'title':'Learning Python',
                     'authors':['Lutz']},
-                [title_test('Learning Python: Powerful Object-Oriented Programming',
+                [title_test('Learning Python, 3rd Edition',
                     exact=True), authors_test(['Mark Lutz'])
                  ]
 
@@ -471,6 +683,38 @@ if __name__ == '__main__':
 
             ),
 
-        ])
+    ] # }}}
 
+    de_tests = [ # {{{
+            (
+                {'identifiers':{'isbn': '3548283519'}},
+                [title_test('Wer Wind sät',
+                    exact=True), authors_test(['Nele Neuhaus'])
+                 ]
+
+            ),
+    ] # }}}
+
+    it_tests = [ # {{{
+            (
+                {'identifiers':{'isbn': '8838922195'}},
+                [title_test('La briscola in cinque',
+                    exact=True), authors_test(['Marco Malvaldi'])
+                 ]
+
+            ),
+    ] # }}}
+
+    fr_tests = [ # {{{
+            (
+                {'identifiers':{'isbn': '2221116798'}},
+                [title_test('L\'étrange voyage de Monsieur Daldry',
+                    exact=True), authors_test(['Marc Levy'])
+                 ]
+
+            ),
+    ] # }}}
+
+    test_identify_plugin(Amazon.name, com_tests)
+# }}}
 
