@@ -226,7 +226,7 @@ class ParseRtf:
         try:
             return_value = process_tokens_obj.process_tokens()
         except InvalidRtfException, msg:
-            #Check to see if the file is correctly encoded
+            # Check to see if the file is correctly encoded
             encode_obj = default_encoding.DefaultEncoding(
             in_file = self.__temp_file,
             run_level = self.__run_level,
@@ -237,14 +237,14 @@ class ParseRtf:
             check_encoding_obj = check_encoding.CheckEncoding(
                     bug_handler = RtfInvalidCodeException,
                         )
-            enc = 'cp' + encode_obj.get_codepage()
-            if enc == 'cp10000':
-                enc = 'mac_roman'
-            msg = 'Exception in token processing'
+            enc = encode_obj.get_codepage()
+            if enc != 'mac_roman':
+                enc = 'cp' + enc
+            msg = '%s\nException in token processing' % str(msg)
             if check_encoding_obj.check_encoding(self.__file, enc):
                 file_name = self.__file if isinstance(self.__file, str) \
                                     else self.__file.encode('utf-8')
-                msg = 'File %s does not appear to be correctly encoded.\n' % file_name
+                msg +='\nFile %s does not appear to be correctly encoded.\n' % file_name
             try:
                 os.remove(self.__temp_file)
             except OSError:
