@@ -9,11 +9,12 @@ from functools import partial
 
 from PyQt4.Qt import QMenu, QObject, QTimer
 
-from calibre.gui2 import error_dialog
+from calibre.gui2 import error_dialog, question_dialog
 from calibre.gui2.dialogs.delete_matching_from_device import DeleteMatchingFromDeviceDialog
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.confirm_delete_location import confirm_location
 from calibre.gui2.actions import InterfaceAction
+from calibre.utils.recycle_bin import can_recycle
 
 single_shot = partial(QTimer.singleShot, 10)
 
@@ -24,6 +25,15 @@ class MultiDeleter(QObject):
         QObject.__init__(self, gui)
         self.model = gui.library_view.model()
         self.ids = ids
+        self.permanent = False
+        if can_recycle and len(ids) > 100:
+            if question_dialog(gui, _('Are you sure?'), '<p>'+
+                _('You are trying to delete %d books. '
+                    'Sending so many files to the Recycle'
+                    ' Bin <b>can be slow</b>. Should calibre skip the'
+                    ' Recycle Bin? If you click Yes the files'
+                    ' will be <b>permanently deleted</b>.')%len(ids)):
+                self.permanent = True
         self.gui = gui
         self.failures = []
         self.deleted_ids = []
@@ -44,7 +54,8 @@ class MultiDeleter(QObject):
             title_ = self.model.db.title(id_, index_is_id=True)
             if title_:
                 title = title_
-            self.model.db.delete_book(id_, notify=False, commit=False)
+            self.model.db.delete_book(id_, notify=False, commit=False,
+                    permanent=self.permanent)
             self.deleted_ids.append(id_)
         except:
             import traceback
