@@ -5,67 +5,155 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import collections, sys
-from Queue import Queue
 
-from PyQt4.Qt import QPixmap, QSize, QWidget, Qt, pyqtSignal, QUrl, \
-    QPropertyAnimation, QEasingCurve, QThread, QApplication, QFontInfo, \
-    QSizePolicy, QPainter, QRect, pyqtProperty, QLayout, QPalette, QMenu
+from PyQt4.Qt import (QPixmap, QSize, QWidget, Qt, pyqtSignal, QUrl,
+    QPropertyAnimation, QEasingCurve, QApplication, QFontInfo,
+    QSizePolicy, QPainter, QRect, pyqtProperty, QLayout, QPalette, QMenu)
 from PyQt4.QtWebKit import QWebView
 
-from calibre import fit_image, prepare_string_for_xml
-from calibre.gui2.dnd import dnd_has_image, dnd_get_image, dnd_get_files, \
-    IMAGE_EXTENSIONS, dnd_has_extension
+from calibre import fit_image, force_unicode, prepare_string_for_xml
+from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
+    IMAGE_EXTENSIONS, dnd_has_extension)
 from calibre.ebooks import BOOK_EXTENSIONS
-from calibre.constants import preferred_encoding
+from calibre.ebooks.metadata.book.base import (field_metadata, Metadata)
+from calibre.ebooks.metadata import fmt_sidx
+from calibre.ebooks.metadata.sources.identify import urls_from_identifiers
+from calibre.constants import filesystem_encoding
 from calibre.library.comments import comments_to_html
-from calibre.gui2 import config, open_local_file, open_url, pixmap_to_data
+from calibre.gui2 import (config, open_local_file, open_url, pixmap_to_data,
+        gprefs)
 from calibre.utils.icu import sort_key
 
-# render_rows(data) {{{
-WEIGHTS = collections.defaultdict(lambda : 100)
-WEIGHTS[_('Path')] = 5
-WEIGHTS[_('Formats')] = 1
-WEIGHTS[_('Collections')] = 2
-WEIGHTS[_('Series')] = 3
-WEIGHTS[_('Tags')] = 4
+def render_html(mi, css, vertical, widget, all_fields=False): # {{{
+    table = render_data(mi, all_fields=all_fields,
+            use_roman_numbers=config['use_roman_numerals_for_series_number'])
 
-def render_rows(data):
-    keys = data.keys()
-    # First sort by name. The WEIGHTS sort will preserve this sub-order
-    keys.sort(key=sort_key)
-    keys.sort(key=lambda x: WEIGHTS[x])
-    rows = []
-    for key in keys:
-        txt = data[key]
-        if key in ('id', _('Comments')) or not hasattr(txt, 'strip') or not txt.strip() or \
-                txt == 'None':
+    def color_to_string(col):
+        ans = '#000000'
+        if col.isValid():
+            col = col.toRgb()
+            if col.isValid():
+                ans = unicode(col.name())
+        return ans
+
+    f = QFontInfo(QApplication.font(widget)).pixelSize()
+    c = color_to_string(QApplication.palette().color(QPalette.Normal,
+                    QPalette.WindowText))
+    templ = u'''\
+    <html>
+        <head>
+        <style type="text/css">
+            body, td {background-color: transparent; font-size: %dpx; color: %s }
+        </style>
+        <style type="text/css">
+            %s
+        </style>
+        </head>
+        <body>
+        %%s
+        </body>
+    <html>
+    '''%(f, c, css)
+    comments = u''
+    if mi.comments:
+        comments = comments_to_html(force_unicode(mi.comments))
+    right_pane = u'<div id="comments" class="comments">%s</div>'%comments
+
+    if vertical:
+        ans = templ%(table+right_pane)
+    else:
+        ans = templ%(u'<table><tr><td valign="top" '
+            'style="padding-right:2em; width:40%%">%s</td><td valign="top">%s</td></tr></table>'
+                % (table, right_pane))
+    return ans
+
+def get_field_list(fm, use_defaults=False):
+    src = gprefs.defaults if use_defaults else gprefs
+    fieldlist = list(src['book_display_fields'])
+    names = frozenset([x[0] for x in fieldlist])
+    for field in fm.displayable_field_keys():
+        if field not in names:
+            fieldlist.append((field, True))
+    return fieldlist
+
+def render_data(mi, use_roman_numbers=True, all_fields=False):
+    ans = []
+    isdevice = not hasattr(mi, 'id')
+    fm = getattr(mi, 'field_metadata', field_metadata)
+
+    for field, display in get_field_list(fm):
+        metadata = fm.get(field, None)
+        if all_fields:
+            display = True
+        if (not display or not metadata or mi.is_null(field) or
+                field == 'comments'):
             continue
-        if isinstance(key, str):
-            key = key.decode(preferred_encoding, 'replace')
-        if isinstance(txt, str):
-            txt = txt.decode(preferred_encoding, 'replace')
-        if key.endswith(u':html'):
-            key = key[:-5]
-            txt = comments_to_html(txt)
-        elif '</font>' not in txt:
-            txt = prepare_string_for_xml(txt)
-        if 'id' in data:
-            if key == _('Path'):
-                txt = u'<a href="path:%s" title="%s">%s</a>'%(data['id'],
-                        txt, _('Click to open'))
-            if key == _('Formats') and txt and txt != _('None'):
-                fmts = [x.strip() for x in txt.split(',')]
-                fmts = [u'<a href="format:%s:%s">%s</a>' % (data['id'], x, x) for x
-                        in fmts]
-                txt = ', '.join(fmts)
+        name = metadata['name']
+        if not name:
+            name = field
+        name += ':'
+        if metadata['datatype'] == 'comments':
+            val = getattr(mi, field)
+            if val:
+                val = force_unicode(val)
+                ans.append((field,
+                    u'<td class="comments" colspan="2">%s</td>'%comments_to_html(val)))
+        elif field == 'path':
+            if mi.path:
+                path = force_unicode(mi.path, filesystem_encoding)
+                scheme = u'devpath' if isdevice else u'path'
+                url = prepare_string_for_xml(path if isdevice else
+                        unicode(mi.id), True)
+                link = u'<a href="%s:%s" title="%s">%s</a>' % (scheme, url,
+                        prepare_string_for_xml(path, True), _('Click to open'))
+                ans.append((field, u'<td class="title">%s</td><td>%s</td>'%(name, link)))
+        elif field == 'formats':
+            if isdevice: continue
+            fmts = [u'<a href="format:%s:%s">%s</a>' % (mi.id, x, x) for x
+                        in mi.formats]
+            ans.append((field, u'<td class="title">%s</td><td>%s</td>'%(name,
+                u', '.join(fmts))))
+        elif field == 'identifiers':
+            urls = urls_from_identifiers(mi.identifiers)
+            links = [u'<a href="%s" title="%s:%s">%s</a>' % (url, id_typ, id_val, name)
+                    for name, id_typ, id_val, url in urls]
+            links = u', '.join(links)
+            if links:
+                ans.append((field, u'<td class="title">%s</td><td>%s</td>'%(
+                    _('Ids')+':', links)))
         else:
-            if key == _('Path'):
-                txt = u'<a href="devpath:%s">%s</a>'%(txt,
-                        _('Click to open'))
+            val = mi.format_field(field)[-1]
+            if val is None:
+                continue
+            val = prepare_string_for_xml(val)
+            if metadata['datatype'] == 'series':
+                sidx = mi.get(field+'_index')
+                if sidx is None:
+                    sidx = 1.0
+                val = _('Book %s of <span class="series_name">%s</span>')%(fmt_sidx(sidx,
+                    use_roman=use_roman_numbers),
+                    prepare_string_for_xml(getattr(mi, field)))
 
-        rows.append((key, txt))
-    return rows
+            ans.append((field, u'<td class="title">%s</td><td>%s</td>'%(name, val)))
+
+    dc = getattr(mi, 'device_collections', [])
+    if dc:
+        dc = u', '.join(sorted(dc, key=sort_key))
+        ans.append(('device_collections',
+            u'<td class="title">%s</td><td>%s</td>'%(
+                _('Collections')+':', dc)))
+
+    def classname(field):
+        try:
+            dt = fm[field]['datatype']
+        except:
+            dt = 'text'
+        return 'datatype_%s'%dt
+
+    ans = [u'<tr id="%s" class="%s">%s</tr>'%(field.replace('#', '_'),
+        classname(field), html) for field, html in ans]
+    # print '\n'.join(ans)
+    return u'<table class="fields">%s</table>'%(u'\n'.join(ans))
 
 # }}}
 
@@ -117,10 +205,10 @@ class CoverView(QWidget): # {{{
 
     def show_data(self, data):
         self.animation.stop()
-        same_item = data.get('id', True) == self.data.get('id', False)
+        same_item = getattr(data, 'id', True) == self.data.get('id', False)
         self.data = {'id':data.get('id', None)}
-        if data.has_key('cover'):
-            self.pixmap = QPixmap.fromImage(data.pop('cover'))
+        if data.cover_data[1]:
+            self.pixmap = QPixmap.fromImage(data.cover_data[1])
             if self.pixmap.isNull() or self.pixmap.width() < 5 or \
                     self.pixmap.height() < 5:
                 self.pixmap = self.default_pixmap
@@ -188,32 +276,6 @@ class CoverView(QWidget): # {{{
 
 # Book Info {{{
 
-class RenderComments(QThread):
-
-    rdone = pyqtSignal(object, object)
-
-    def __init__(self, parent):
-        QThread.__init__(self, parent)
-        self.queue = Queue()
-        self.start()
-
-    def run(self):
-        while True:
-            try:
-                rows, comments = self.queue.get()
-            except:
-                break
-            import time
-            time.sleep(0.001)
-            oint = sys.getcheckinterval()
-            sys.setcheckinterval(5)
-            try:
-                self.rdone.emit(rows, comments_to_html(comments))
-            except:
-                pass
-            sys.setcheckinterval(oint)
-
-
 class BookInfo(QWebView):
 
     link_clicked = pyqtSignal(object)
@@ -221,8 +283,6 @@ class BookInfo(QWebView):
     def __init__(self, vertical, parent=None):
         QWebView.__init__(self, parent)
         self.vertical = vertical
-        self.renderer = RenderComments(self)
-        self.renderer.rdone.connect(self._show_data, type=Qt.QueuedConnection)
         self.page().setLinkDelegationPolicy(self.page().DelegateAllLinks)
         self.linkClicked.connect(self.link_activated)
         self._link_clicked = False
@@ -231,62 +291,21 @@ class BookInfo(QWebView):
         self.setAcceptDrops(False)
         palette.setBrush(QPalette.Base, Qt.transparent)
         self.page().setPalette(palette)
+        self.css = P('templates/book_details.css', data=True).decode('utf-8')
 
     def link_activated(self, link):
         self._link_clicked = True
+        if unicode(link.scheme()) in ('http', 'https'):
+            return open_url(link)
         link = unicode(link.toString())
         self.link_clicked.emit(link)
 
     def turnoff_scrollbar(self, *args):
         self.page().mainFrame().setScrollBarPolicy(Qt.Horizontal, Qt.ScrollBarAlwaysOff)
 
-    def show_data(self, data):
-        rows = render_rows(data)
-        rows = u'\n'.join([u'<tr><td valign="top"><b>%s:</b></td><td valign="top">%s</td></tr>'%(k,t) for
-            k, t in rows])
-        comments = data.get(_('Comments'), '')
-        if not comments or comments == u'None':
-            comments = ''
-        self.renderer.queue.put((rows, comments))
-        self._show_data(rows, '')
-
-
-    def _show_data(self, rows, comments):
-
-        def color_to_string(col):
-            ans = '#000000'
-            if col.isValid():
-                col = col.toRgb()
-                if col.isValid():
-                    ans = unicode(col.name())
-            return ans
-
-        f = QFontInfo(QApplication.font(self.parent())).pixelSize()
-        c = color_to_string(QApplication.palette().color(QPalette.Normal,
-                        QPalette.WindowText))
-        templ = u'''\
-        <html>
-            <head>
-            <style type="text/css">
-                body, td {background-color: transparent; font-size: %dpx; color: %s }
-                a { text-decoration: none; color: blue }
-            </style>
-            </head>
-            <body>
-            %%s
-            </body>
-        <html>
-        '''%(f, c)
-        if self.vertical:
-            if comments:
-                rows += u'<tr><td colspan="2">%s</td></tr>'%comments
-            self.setHtml(templ%(u'<table>%s</table>'%rows))
-        else:
-            left_pane = u'<table>%s</table>'%rows
-            right_pane = u'<div>%s</div>'%comments
-            self.setHtml(templ%(u'<table><tr><td valign="top" '
-                'style="padding-right:2em; width:40%%">%s</td><td valign="top">%s</td></tr></table>'
-                    % (left_pane, right_pane)))
+    def show_data(self, mi):
+        html = render_html(mi, self.css, self.vertical, self.parent())
+        self.setHtml(html)
 
     def mouseDoubleClickEvent(self, ev):
         swidth = self.page().mainFrame().scrollBarGeometry(Qt.Vertical).width()
@@ -415,6 +434,7 @@ class BookDetails(QWidget): # {{{
             if y is None:
                 # Local image
                 self.cover_view.paste_from_clipboard(x)
+                self.update_layout()
             else:
                 self.remote_file_dropped.emit(x, y)
                 # We do not support setting cover *and* adding formats for
@@ -446,16 +466,17 @@ class BookDetails(QWidget): # {{{
         self.setAcceptDrops(True)
         self._layout = DetailsLayout(vertical, self)
         self.setLayout(self._layout)
+        self.current_path = ''
 
         self.cover_view = CoverView(vertical, self)
         self.cover_view.cover_changed.connect(self.cover_changed.emit)
         self._layout.addWidget(self.cover_view)
         self.book_info = BookInfo(vertical, self)
         self._layout.addWidget(self.book_info)
-        self.book_info.link_clicked.connect(self._link_clicked)
+        self.book_info.link_clicked.connect(self.handle_click)
         self.setCursor(Qt.PointingHandCursor)
 
-    def _link_clicked(self, link):
+    def handle_click(self, link):
         typ, _, val = link.partition(':')
         if typ == 'path':
             self.open_containing_folder.emit(int(val))
@@ -479,12 +500,23 @@ class BookDetails(QWidget): # {{{
     def show_data(self, data):
         self.book_info.show_data(data)
         self.cover_view.show_data(data)
+        self.current_path = getattr(data, u'path', u'')
+        self.update_layout()
+
+    def update_layout(self):
         self._layout.do_layout(self.rect())
-        self.setToolTip('<p>'+_('Double-click to open Book Details window') +
-                '<br><br>' + _('Path') + ': ' + data.get(_('Path'), ''))
+        try:
+            sz = self.cover_view.pixmap.size()
+        except:
+            sz = QSize(0, 0)
+        self.setToolTip(
+            '<p>'+_('Double-click to open Book Details window') +
+            '<br><br>' + _('Path') + ': ' + self.current_path +
+            '<br><br>' + _('Cover size: %dx%d')%(sz.width(), sz.height())
+        )
 
     def reset_info(self):
-        self.show_data({})
+        self.show_data(Metadata(_('Unknown')))
 
 # }}}
 
