@@ -9,6 +9,7 @@ Scheduler for automated recipe downloads
 
 from datetime import timedelta
 import calendar, textwrap
+from collections import OrderedDict
 
 from PyQt4.Qt import QDialog, Qt, QTime, QObject, QMenu, QHBoxLayout, \
         QAction, QIcon, QMutex, QTimer, pyqtSignal, QWidget, QGridLayout, \
@@ -20,7 +21,6 @@ from calibre.web.feeds.recipes.model import RecipeModel
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.date import utcnow
 from calibre.utils.network import internet_connected
-from calibre.utils.ordered_dict import OrderedDict
 from calibre import force_unicode
 
 def convert_day_time_schedule(val):
@@ -68,7 +68,7 @@ class DaysOfWeek(Base):
     def initialize(self, typ=None, val=None):
         if typ is None:
             typ = 'day/time'
-            val = (-1, 9, 0)
+            val = (-1, 6, 0)
         if typ == 'day/time':
             val = convert_day_time_schedule(val)
 
@@ -118,7 +118,7 @@ class DaysOfMonth(Base):
 
     def initialize(self, typ=None, val=None):
         if val is None:
-            val = ((1,), 9, 0)
+            val = ((1,), 6, 0)
         days_of_month, hour, minute = val
         self.days.setText(', '.join(map(str, map(int, days_of_month))))
         self.time.setTime(QTime(hour, minute))
@@ -380,7 +380,7 @@ class SchedulerDialog(QDialog, Ui_Dialog):
             if d < timedelta(days=366):
                 ld_text = tm
         else:
-            typ, sch = 'day/time', (-1, 9, 0)
+            typ, sch = 'day/time', (-1, 6, 0)
         sch_widget = {'day/time': 0, 'days_of_week': 0, 'days_of_month':1,
                 'interval':2}[typ]
         rb = getattr(self, list(self.SCHEDULE_TYPES)[sch_widget])
