@@ -7,23 +7,21 @@ __docformat__ = 'restructuredtext en'
 
 import shutil, functools, re, os, traceback
 from contextlib import closing
-from operator import attrgetter
 
-from PyQt4.Qt import QAbstractTableModel, Qt, pyqtSignal, QIcon, QImage, \
-        QModelIndex, QVariant, QDate, QColor
+from PyQt4.Qt import (QAbstractTableModel, Qt, pyqtSignal, QIcon, QImage,
+        QModelIndex, QVariant, QDate, QColor)
 
-from calibre.gui2 import NONE, config, UNDEFINED_QDATE
+from calibre.gui2 import NONE, UNDEFINED_QDATE
 from calibre.utils.pyparsing import ParseException
 from calibre.ebooks.metadata import fmt_sidx, authors_to_string, string_to_authors
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.config import tweaks, prefs
-from calibre.utils.date import dt_factory, qt_to_dt, isoformat
-from calibre.utils.icu import sort_key, strcmp as icu_strcmp
-from calibre.ebooks.metadata.meta import set_metadata as _set_metadata
+from calibre.utils.date import dt_factory, qt_to_dt
+from calibre.utils.icu import sort_key
 from calibre.utils.search_query_parser import SearchQueryParser
-from calibre.library.caches import _match, CONTAINS_MATCH, EQUALS_MATCH, \
-    REGEXP_MATCH, MetadataBackup, force_to_bool
-from calibre import strftime, isbytestring, prepare_string_for_xml
+from calibre.library.caches import (_match, CONTAINS_MATCH, EQUALS_MATCH,
+    REGEXP_MATCH, MetadataBackup, force_to_bool)
+from calibre import strftime, isbytestring
 from calibre.constants import filesystem_encoding, DEBUG
 from calibre.gui2.library import DEFAULT_SORT
 
@@ -72,6 +70,7 @@ class BooksModel(QAbstractTableModel): # {{{
                         'publisher' : _("Publisher"),
                         'tags'      : _("Tags"),
                         'series'    : _("Series"),
+                        'last_modified' : _('Modified'),
     }
 
     def __init__(self, parent=None, buffer=40):
@@ -116,7 +115,7 @@ class BooksModel(QAbstractTableModel): # {{{
         return cc_label in self.custom_columns
 
     def read_config(self):
-        self.use_roman_numbers = config['use_roman_numerals_for_series_number']
+        pass
 
     def set_device_connected(self, is_connected):
         self.device_connected = is_connected
@@ -311,10 +310,17 @@ class BooksModel(QAbstractTableModel): # {{{
     def sort(self, col, order, reset=True):
         if not self.db:
             return
-        self.about_to_be_sorted.emit(self.db.id)
         if not isinstance(order, bool):
             order = order == Qt.AscendingOrder
         label = self.column_map[col]
+        self._sort(label, order, reset)
+
+    def sort_by_named_field(self, field, order, reset=True):
+        if field in self.db.field_metadata.keys():
+            self._sort(field, order, reset)
+
+    def _sort(self, label, order, reset):
+        self.about_to_be_sorted.emit(self.db.id)
         self.db.sort(label, order)
         if reset:
             self.reset()
@@ -350,63 +356,13 @@ class BooksModel(QAbstractTableModel): # {{{
         return self.rowCount(None)
 
     def get_book_display_info(self, idx):
-        def custom_keys_to_display():
-            ans = getattr(self, '_custom_fields_in_book_info', None)
-            if ans is None:
-                cfkeys = set(self.db.custom_field_keys())
-                yes_fields = set(tweaks['book_details_will_display'])
-                no_fields = set(tweaks['book_details_wont_display'])
-                if '*' in yes_fields:
-                    yes_fields = cfkeys
-                if '*' in no_fields:
-                    no_fields = cfkeys
-                ans = frozenset(yes_fields - no_fields)
-                setattr(self, '_custom_fields_in_book_info', ans)
-            return ans
-
-        data = {}
-        cdata = self.cover(idx)
-        if cdata:
-            data['cover'] = cdata
-        tags = list(self.db.get_tags(self.db.id(idx)))
-        if tags:
-            tags.sort(key=sort_key)
-            tags = ', '.join(tags)
-        else:
-            tags = _('None')
-        data[_('Tags')] = tags
-        formats = self.db.formats(idx)
-        if formats:
-            formats = formats.replace(',', ', ')
-        else:
-            formats = _('None')
-        data[_('Formats')] = formats
-        data[_('Path')] = self.db.abspath(idx)
-        data['id'] = self.id(idx)
-        comments = self.db.comments(idx)
-        if not comments:
-            comments = _('None')
-        data[_('Comments')] = comments
-        series = self.db.series(idx)
-        if series:
-            sidx = self.db.series_index(idx)
-            sidx = fmt_sidx(sidx, use_roman = self.use_roman_numbers)
-            data[_('Series')] = \
-                _('Book %s of %s.')%\
-                    (sidx, prepare_string_for_xml(series))
         mi = self.db.get_metadata(idx)
-        cf_to_display = custom_keys_to_display()
-        for key in mi.custom_field_keys():
-            if key not in cf_to_display:
-                continue
-            name, val = mi.format_field(key)
-            if mi.metadata_for_field(key)['datatype'] == 'comments':
-                name += ':html'
-            if val and name not in data:
-                data[name] = val
-
-        return data
-
+        mi.size = mi.book_size
+        mi.cover_data = ('jpg', self.cover(idx))
+        mi.id = self.db.id(idx)
+        mi.field_metadata = self.db.field_metadata
+        mi.path = self.db.abspath(idx, create_dirs=False)
+        return mi
 
     def current_changed(self, current, previous, emit_signal=True):
         if current.isValid():
@@ -420,16 +376,8 @@ class BooksModel(QAbstractTableModel): # {{{
     def get_book_info(self, index):
         if isinstance(index, int):
             index = self.index(index, 0)
+        # If index is not valid returns None
         data = self.current_changed(index, None, False)
-        if data is None:
-            return data
-        row = index.row()
-        data[_('Title')] = self.db.title(row)
-        au = self.db.authors(row)
-        if not au:
-            au = _('Unknown')
-        au = ', '.join([a.strip() for a in au.split(',')])
-        data[_('Author(s)')] = au
         return data
 
     def metadata_for(self, ids):
@@ -472,6 +420,7 @@ class BooksModel(QAbstractTableModel): # {{{
     def get_preferred_formats_from_ids(self, ids, formats,
                               set_metadata=False, specific_format=None,
                               exclude_auto=False, mode='r+b'):
+        from calibre.ebooks.metadata.meta import set_metadata as _set_metadata
         ans = []
         need_auto = []
         if specific_format is not None:
@@ -520,6 +469,7 @@ class BooksModel(QAbstractTableModel): # {{{
     def get_preferred_formats(self, rows, formats, paths=False,
                               set_metadata=False, specific_format=None,
                               exclude_auto=False):
+        from calibre.ebooks.metadata.meta import set_metadata as _set_metadata
         ans = []
         need_auto = []
         if specific_format is not None:
@@ -556,6 +506,9 @@ class BooksModel(QAbstractTableModel): # {{{
     def id(self, row):
         return self.db.id(getattr(row, 'row', lambda:row)())
 
+    def authors(self, row_number):
+        return self.db.authors(row_number)
+
     def title(self, row_number):
         return self.db.title(row_number)
 
@@ -605,7 +558,10 @@ class BooksModel(QAbstractTableModel): # {{{
         def size(r, idx=-1):
             size = self.db.data[r][idx]
             if size:
-                return QVariant('%.1f'%(float(size)/(1024*1024)))
+                ans = '%.1f'%(float(size)/(1024*1024))
+                if size > 0 and ans == '0.0':
+                    ans = '<0.1'
+                return QVariant(ans)
             return None
 
         def rating_type(r, idx=-1):
@@ -640,18 +596,18 @@ class BooksModel(QAbstractTableModel): # {{{
                 return self.bool_yes_icon
             return self.bool_blank_icon
 
-        def text_type(r, mult=False, idx=-1):
+        def text_type(r, mult=None, idx=-1):
             text = self.db.data[r][idx]
-            if text and mult:
-                return QVariant(', '.join(sorted(text.split('|'),key=sort_key)))
+            if text and mult is not None:
+                if mult:
+                    return QVariant(u' & '.join(text.split('|')))
+                return QVariant(u', '.join(sorted(text.split('|'),key=sort_key)))
             return QVariant(text)
 
-        def decorated_text_type(r, mult=False, idx=-1):
+        def decorated_text_type(r, idx=-1):
             text = self.db.data[r][idx]
             if force_to_bool(text) is not None:
                 return None
-            if text and mult:
-                return QVariant(', '.join(sorted(text.split('|'),key=sort_key)))
             return QVariant(text)
 
         def number_type(r, idx=-1):
@@ -659,7 +615,7 @@ class BooksModel(QAbstractTableModel): # {{{
 
         self.dc = {
                    'title'    : functools.partial(text_type,
-                                idx=self.db.field_metadata['title']['rec_index'], mult=False),
+                                idx=self.db.field_metadata['title']['rec_index'], mult=None),
                    'authors'  : functools.partial(authors,
                                 idx=self.db.field_metadata['authors']['rec_index']),
                    'size'     : functools.partial(size,
@@ -668,17 +624,19 @@ class BooksModel(QAbstractTableModel): # {{{
                                 idx=self.db.field_metadata['timestamp']['rec_index']),
                    'pubdate'  : functools.partial(datetime_type,
                                 idx=self.db.field_metadata['pubdate']['rec_index']),
+                   'last_modified': functools.partial(datetime_type,
+                                idx=self.db.field_metadata['last_modified']['rec_index']),
                    'rating'   : functools.partial(rating_type,
                                 idx=self.db.field_metadata['rating']['rec_index']),
                    'publisher': functools.partial(text_type,
-                                idx=self.db.field_metadata['publisher']['rec_index'], mult=False),
+                                idx=self.db.field_metadata['publisher']['rec_index'], mult=None),
                    'tags'     : functools.partial(tags,
                                 idx=self.db.field_metadata['tags']['rec_index']),
                    'series'   : functools.partial(series_type,
                                 idx=self.db.field_metadata['series']['rec_index'],
                                 siix=self.db.field_metadata['series_index']['rec_index']),
                    'ondevice' : functools.partial(text_type,
-                                idx=self.db.field_metadata['ondevice']['rec_index'], mult=False),
+                                idx=self.db.field_metadata['ondevice']['rec_index'], mult=None),
                    }
 
         self.dc_decorator = {
@@ -692,15 +650,16 @@ class BooksModel(QAbstractTableModel): # {{{
             datatype = self.custom_columns[col]['datatype']
             if datatype in ('text', 'comments', 'composite', 'enumeration'):
                 mult=self.custom_columns[col]['is_multiple']
+                if mult is not None:
+                    mult = self.custom_columns[col]['display'].get('is_names', False)
                 self.dc[col] = functools.partial(text_type, idx=idx, mult=mult)
                 if datatype in ['text', 'composite', 'enumeration'] and not mult:
                     if self.custom_columns[col]['display'].get('use_decorations', False):
-                        self.dc[col] = functools.partial(decorated_text_type,
-                                                         idx=idx, mult=mult)
+                        self.dc[col] = functools.partial(decorated_text_type, idx=idx)
                         self.dc_decorator[col] = functools.partial(
                             bool_type_decorator, idx=idx,
                             bool_cols_are_tristate=
-                                tweaks['bool_custom_columns_are_tristate'] != 'no')
+                                self.db.prefs.get('bools_are_tristate'))
             elif datatype in ('int', 'float'):
                 self.dc[col] = functools.partial(number_type, idx=idx)
             elif datatype == 'datetime':
@@ -710,7 +669,7 @@ class BooksModel(QAbstractTableModel): # {{{
                 self.dc_decorator[col] = functools.partial(
                             bool_type_decorator, idx=idx,
                             bool_cols_are_tristate=
-                                tweaks['bool_custom_columns_are_tristate'] != 'no')
+                                self.db.prefs.get('bools_are_tristate'))
             elif datatype == 'rating':
                 self.dc[col] = functools.partial(rating_type, idx=idx)
             elif datatype == 'series':
@@ -983,6 +942,21 @@ class OnDeviceSearch(SearchQueryParser): # {{{
 
 # }}}
 
+class DeviceDBSortKeyGen(object): # {{{
+
+    def __init__(self, attr, keyfunc, db):
+        self.attr = attr
+        self.db = db
+        self.keyfunc = keyfunc
+
+    def __call__(self, x):
+        try:
+            ans = self.keyfunc(getattr(self.db[x], self.attr))
+        except:
+            ans = None
+        return ans
+# }}}
+
 class DeviceBooksModel(BooksModel): # {{{
 
     booklist_dirtied = pyqtSignal()
@@ -1088,59 +1062,40 @@ class DeviceBooksModel(BooksModel): # {{{
 
     def sort(self, col, order, reset=True):
         descending = order != Qt.AscendingOrder
-        def strcmp(attr):
-            ag = attrgetter(attr)
-            def _strcmp(x, y):
-                x = ag(self.db[x])
-                y = ag(self.db[y])
-                if x == None:
-                    x = ''
-                if y == None:
-                    y = ''
-                return icu_strcmp(x.strip(), y.strip())
-            return _strcmp
-        def datecmp(x, y):
-            x = self.db[x].datetime
-            y = self.db[y].datetime
-            return cmp(dt_factory(x, assume_utc=True), dt_factory(y,
-                assume_utc=True))
-        def sizecmp(x, y):
-            x, y = int(self.db[x].size), int(self.db[y].size)
-            return cmp(x, y)
-        def tagscmp(x, y):
-            x = ','.join(sorted(getattr(self.db[x], 'device_collections', []),key=sort_key))
-            y = ','.join(sorted(getattr(self.db[y], 'device_collections', []),key=sort_key))
-            return cmp(x, y)
-        def libcmp(x, y):
-            x, y = self.db[x].in_library, self.db[y].in_library
-            return cmp(x, y)
-        def authorcmp(x, y):
-            ax = getattr(self.db[x], 'author_sort', None)
-            ay = getattr(self.db[y], 'author_sort', None)
-            if ax and ay:
-                x = ax
-                y = ay
-            else:
-                x, y = authors_to_string(self.db[x].authors), \
-                                authors_to_string(self.db[y].authors)
-            return cmp(x, y)
         cname = self.column_map[col]
-        fcmp = {
-                'title': strcmp('title_sorter'),
-                'authors' : authorcmp,
-                'size' : sizecmp,
-                'timestamp': datecmp,
-                'collections': tagscmp,
-                'inlibrary': libcmp,
+        def author_key(x):
+            try:
+                ax = self.db[x].author_sort
+                if not ax:
+                    raise Exception('')
+            except:
+                try:
+                    ax = authors_to_string(self.db[x].authors)
+                except:
+                    ax = ''
+            return ax
+
+        keygen = {
+                'title': ('title_sorter', lambda x: sort_key(x) if x else ''),
+                'authors' : author_key,
+                'size' : ('size', int),
+                'timestamp': ('datetime', functools.partial(dt_factory, assume_utc=True)),
+                'collections': ('device_collections', lambda x:sorted(x,
+                    key=sort_key)),
+                'inlibrary': ('in_library', lambda x: x),
                 }[cname]
-        self.map.sort(cmp=fcmp, reverse=descending)
+        keygen = keygen if callable(keygen) else DeviceDBSortKeyGen(
+            keygen[0], keygen[1], self.db)
+        self.map.sort(key=keygen, reverse=descending)
         if len(self.map) == len(self.db):
             self.sorted_map = list(self.map)
         else:
             self.sorted_map = list(range(len(self.db)))
-            self.sorted_map.sort(cmp=fcmp, reverse=descending)
+            self.sorted_map.sort(key=keygen, reverse=descending)
         self.sorted_on = (self.column_map[col], order)
         self.sort_history.insert(0, self.sorted_on)
+        if hasattr(keygen, 'db'):
+            keygen.db = None
         if reset:
             self.reset()
 
@@ -1182,39 +1137,46 @@ class DeviceBooksModel(BooksModel): # {{{
             img = self.default_image
         return img
 
-    def current_changed(self, current, previous):
-        data = {}
-        item = self.db[self.map[current.row()]]
-        cover = self.cover(current.row())
-        if cover is not self.default_image:
-            data['cover'] = cover
-        type = _('Unknown')
+    def get_book_display_info(self, idx):
+        from calibre.ebooks.metadata.book.base import Metadata
+        item = self.db[self.map[idx]]
+        cover = self.cover(idx)
+        if cover is self.default_image:
+            cover = None
+        title = item.title
+        if not title:
+            title = _('Unknown')
+        au = item.authors
+        if not au:
+            au = [_('Unknown')]
+        mi = Metadata(title, au)
+        mi.cover_data = ('jpg', cover)
+        fmt = _('Unknown')
         ext = os.path.splitext(item.path)[1]
         if ext:
-            type = ext[1:].lower()
-        data[_('Format')] = type
-        data[_('Path')] = item.path
+            fmt = ext[1:].lower()
+        mi.formats = [fmt]
+        mi.path = (item.path if item.path else None)
         dt = dt_factory(item.datetime, assume_utc=True)
-        data[_('Timestamp')] = isoformat(dt, sep=' ', as_utc=False)
-        data[_('Collections')] = ', '.join(item.device_collections)
-
-        tags = getattr(item, 'tags', None)
-        if tags:
-            tags = u', '.join(tags)
-        else:
-            tags = _('None')
-        data[_('Tags')] = tags
-        comments = getattr(item, 'comments', None)
-        if not comments:
-            comments = _('None')
-        data[_('Comments')] = comments
+        mi.timestamp = dt
+        mi.device_collections = list(item.device_collections)
+        mi.tags = list(getattr(item, 'tags', []))
+        mi.comments = getattr(item, 'comments', None)
         series = getattr(item, 'series', None)
         if series:
             sidx = getattr(item, 'series_index', 0)
-            sidx = fmt_sidx(sidx, use_roman = self.use_roman_numbers)
-            data[_('Series')] = _('Book <font face="serif">%s</font> of %s.')%(sidx, series)
+            mi.series = series
+            mi.series_index = sidx
+        return mi
 
-        self.new_bookdisplay_data.emit(data)
+    def current_changed(self, current, previous, emit_signal=True):
+        if current.isValid():
+            idx = current.row()
+            data = self.get_book_display_info(idx)
+            if emit_signal:
+                self.new_bookdisplay_data.emit(data)
+            else:
+                return data
 
     def paths(self, rows):
         return [self.db[self.map[r.row()]].path for r in rows ]
@@ -1274,7 +1236,7 @@ class DeviceBooksModel(BooksModel): # {{{
             elif cname == 'authors':
                 au = self.db[self.map[row]].authors
                 if not au:
-                    au = self.unknown
+                    au = [_('Unknown')]
                 return QVariant(authors_to_string(au))
             elif cname == 'size':
                 size = self.db[self.map[row]].size
