@@ -1,5 +1,7 @@
 #!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+from __future__ import (unicode_literals, division, absolute_import,
+                        print_function)
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -8,31 +10,34 @@ __docformat__ = 'restructuredtext en'
 import os
 from functools import partial
 
-from PyQt4.Qt import Qt, QVBoxLayout, QHBoxLayout, QWidget, QPushButton, \
-        QGridLayout, pyqtSignal, QDialogButtonBox, QScrollArea, QFont, \
-        QTabWidget, QIcon, QToolButton, QSplitter, QGroupBox, QSpacerItem, \
-        QSizePolicy, QPalette, QFrame, QSize, QKeySequence
+from PyQt4.Qt import (Qt, QVBoxLayout, QHBoxLayout, QWidget, QPushButton,
+        QGridLayout, pyqtSignal, QDialogButtonBox, QScrollArea, QFont,
+        QTabWidget, QIcon, QToolButton, QSplitter, QGroupBox, QSpacerItem,
+        QSizePolicy, QPalette, QFrame, QSize, QKeySequence, QMenu)
 
 from calibre.ebooks.metadata import authors_to_string, string_to_authors
-from calibre.gui2 import ResizableDialog, error_dialog, gprefs
-from calibre.gui2.metadata.basic_widgets import TitleEdit, AuthorsEdit, \
-    AuthorSortEdit, TitleSortEdit, SeriesEdit, SeriesIndexEdit, ISBNEdit, \
-    RatingEdit, PublisherEdit, TagsEdit, FormatsManager, Cover, CommentsEdit, \
-    BuddyLabel, DateEdit, PubdateEdit
+from calibre.gui2 import ResizableDialog, error_dialog, gprefs, pixmap_to_data
+from calibre.gui2.metadata.basic_widgets import (TitleEdit, AuthorsEdit,
+    AuthorSortEdit, TitleSortEdit, SeriesEdit, SeriesIndexEdit, IdentifiersEdit,
+    RatingEdit, PublisherEdit, TagsEdit, FormatsManager, Cover, CommentsEdit,
+    BuddyLabel, DateEdit, PubdateEdit)
+from calibre.gui2.metadata.single_download import FullFetch
 from calibre.gui2.custom_column_widgets import populate_metadata_page
 from calibre.utils.config import tweaks
+from calibre.ebooks.metadata.book.base import Metadata
 
 class MetadataSingleDialogBase(ResizableDialog):
 
-    view_format = pyqtSignal(object)
+    view_format = pyqtSignal(object, object)
     cc_two_column = tweaks['metadata_single_use_2_cols_for_custom_fields']
     one_line_comments_toolbar = False
+    use_toolbutton_for_config_metadata = True
 
     def __init__(self, db, parent=None):
         self.db = db
-        self.changed = set([])
-        self.books_to_refresh = set([])
-        self.rows_to_refresh = set([])
+        self.changed = set()
+        self.books_to_refresh = set()
+        self.rows_to_refresh = set()
         ResizableDialog.__init__(self, parent)
 
     def setupUi(self, *args): # {{{
@@ -65,7 +70,11 @@ class MetadataSingleDialogBase(ResizableDialog):
         self.setLayout(self.l)
         self.l.setMargin(0)
         self.l.addWidget(self.scroll_area)
-        self.l.addWidget(self.button_box)
+        ll = self.button_box_layout = QHBoxLayout()
+        self.l.addLayout(ll)
+        ll.addSpacing(10)
+        ll.addWidget(self.button_box)
+        ll.addSpacing(10)
 
         self.setWindowIcon(QIcon(I('edit_input.png')))
         self.setWindowTitle(_('Edit Metadata'))
@@ -98,15 +107,21 @@ class MetadataSingleDialogBase(ResizableDialog):
                 self.deduce_title_sort_button)
         self.basic_metadata_widgets.extend([self.title, self.title_sort])
 
-        self.authors = AuthorsEdit(self)
-        self.deduce_author_sort_button = QToolButton(self)
-        self.deduce_author_sort_button.setToolTip(_(
-        'Automatically create the author sort entry based on the current'
-        ' author entry.\n'
-        'Using this button to create author sort will change author sort from'
-        ' red to green.'))
-        self.author_sort = AuthorSortEdit(self, self.authors,
-                self.deduce_author_sort_button, self.db)
+        self.deduce_author_sort_button = b = QToolButton(self)
+        b.setToolTip('<p>' +
+            _('Automatically create the author sort entry based on the current '
+              'author entry. Using this button to create author sort will '
+              'change author sort from red to green.  There is a menu of '
+              'functions available under this button. Click and hold '
+              'on the button to see it.') + '</p>')
+        b.m = m = QMenu()
+        ac = m.addAction(QIcon(I('forward.png')), _('Set author sort from author'))
+        ac2 = m.addAction(QIcon(I('back.png')), _('Set author from author sort'))
+        ac3 = m.addAction(QIcon(I('user_profile.png')), _('Manage authors'))
+        b.setMenu(m)
+        self.authors = AuthorsEdit(self, ac3)
+        self.author_sort = AuthorSortEdit(self, self.authors, b, self.db, ac,
+                ac2)
         self.basic_metadata_widgets.extend([self.authors, self.author_sort])
 
         self.swap_title_author_button = QToolButton(self)
@@ -115,6 +130,13 @@ class MetadataSingleDialogBase(ResizableDialog):
             'Swap the author and title'))
         self.swap_title_author_button.clicked.connect(self.swap_title_author)
 
+        self.manage_authors_button = QToolButton(self)
+        self.manage_authors_button.setIcon(QIcon(I('user_profile.png')))
+        self.manage_authors_button.setToolTip('<p>' + _(
+            'Manage authors. Use to rename authors and correct '
+            'individual author\'s sort values') + '</p>')
+        self.manage_authors_button.clicked.connect(self.authors.manage_authors)
+
         self.series = SeriesEdit(self)
         self.remove_unused_series_button = QToolButton(self)
         self.remove_unused_series_button.setToolTip(
@@ -130,6 +152,7 @@ class MetadataSingleDialogBase(ResizableDialog):
         self.formats_manager.cover_from_format_button.clicked.connect(
                 self.cover_from_format)
         self.cover = Cover(self)
+        self.cover.download_cover.connect(self.download_cover)
         self.basic_metadata_widgets.append(self.cover)
 
         self.comments = CommentsEdit(self, self.one_line_comments_toolbar)
@@ -145,8 +168,17 @@ class MetadataSingleDialogBase(ResizableDialog):
         self.tags_editor_button.clicked.connect(self.tags_editor)
         self.basic_metadata_widgets.append(self.tags)
 
-        self.isbn = ISBNEdit(self)
-        self.basic_metadata_widgets.append(self.isbn)
+        self.identifiers = IdentifiersEdit(self)
+        self.basic_metadata_widgets.append(self.identifiers)
+        self.clear_identifiers_button = QToolButton(self)
+        self.clear_identifiers_button.setIcon(QIcon(I('trash.png')))
+        self.clear_identifiers_button.clicked.connect(self.identifiers.clear)
+        self.paste_isbn_button = QToolButton(self)
+        self.paste_isbn_button.setToolTip('<p>' +
+                    _('Paste the contents of the clipboard into the '
+                      'identifiers box prefixed with isbn:') + '</p>')
+        self.paste_isbn_button.setIcon(QIcon(I('edit-paste.png')))
+        self.paste_isbn_button.clicked.connect(self.identifiers.paste_isbn)
 
         self.publisher = PublisherEdit(self)
         self.basic_metadata_widgets.append(self.publisher)
@@ -156,12 +188,22 @@ class MetadataSingleDialogBase(ResizableDialog):
         self.basic_metadata_widgets.extend([self.timestamp, self.pubdate])
 
         self.fetch_metadata_button = QPushButton(
-                _('&Fetch metadata from server'), self)
+                _('&Download metadata'), self)
         self.fetch_metadata_button.clicked.connect(self.fetch_metadata)
         font = self.fmb_font = QFont()
         font.setBold(True)
         self.fetch_metadata_button.setFont(font)
 
+        if self.use_toolbutton_for_config_metadata:
+            self.config_metadata_button = QToolButton(self)
+            self.config_metadata_button.setIcon(QIcon(I('config.png')))
+        else:
+            self.config_metadata_button = QPushButton(self)
+            self.config_metadata_button.setText(_('Configure download metadata'))
+        self.config_metadata_button.setIcon(QIcon(I('config.png')))
+        self.config_metadata_button.clicked.connect(self.configure_metadata)
+        self.config_metadata_button.setToolTip(
+            _('Change how calibre downloads metadata'))
 
     # }}}
 
@@ -181,7 +223,7 @@ class MetadataSingleDialogBase(ResizableDialog):
             ans = self.custom_metadata_widgets
             for i in range(len(ans)-1):
                 if before is not None and i == 0:
-                    pass# Do something
+                    pass
                 if len(ans[i+1].widgets) == 2:
                     sto(ans[i].widgets[-1], ans[i+1].widgets[1])
                 else:
@@ -189,9 +231,16 @@ class MetadataSingleDialogBase(ResizableDialog):
                 for c in range(2, len(ans[i].widgets), 2):
                     sto(ans[i].widgets[c-1], ans[i].widgets[c+1])
             if after is not None:
-                pass # Do something
+                pass
     # }}}
 
+    def do_view_format(self, path, fmt):
+        if path:
+            self.view_format.emit(None, path)
+        else:
+            self.view_format.emit(self.book_id, fmt)
+
+
     def do_layout(self):
         raise NotImplementedError()
 
@@ -202,6 +251,8 @@ class MetadataSingleDialogBase(ResizableDialog):
             widget.initialize(self.db, id_)
         for widget in getattr(self, 'custom_metadata_widgets', []):
             widget.initialize(id_)
+        if callable(self.set_current_callback):
+            self.set_current_callback(id_)
         # Commented out as it doesn't play nice with Next, Prev buttons
         #self.fetch_metadata_button.setFocus(Qt.OtherFocusReason)
 
@@ -264,13 +315,17 @@ class MetadataSingleDialogBase(ResizableDialog):
                          show=True)
             return
 
-    def update_from_mi(self, mi):
+    def update_from_mi(self, mi, update_sorts=True):
         if not mi.is_null('title'):
             self.title.current_val = mi.title
+            if update_sorts:
+                self.title_sort.auto_generate()
         if not mi.is_null('authors'):
             self.authors.current_val = mi.authors
         if not mi.is_null('author_sort'):
             self.author_sort.current_val = mi.author_sort
+        elif update_sorts:
+            self.author_sort.auto_generate()
         if not mi.is_null('rating'):
             try:
                 self.rating.current_val = mi.rating
@@ -280,8 +335,10 @@ class MetadataSingleDialogBase(ResizableDialog):
             self.publisher.current_val = mi.publisher
         if not mi.is_null('tags'):
             self.tags.current_val = mi.tags
-        if not mi.is_null('isbn'):
-            self.isbn.current_val = mi.isbn
+        if not mi.is_null('identifiers'):
+            current = self.identifiers.current_val
+            current.update(mi.identifiers)
+            self.identifiers.current_val = current
         if not mi.is_null('pubdate'):
             self.pubdate.current_val = mi.pubdate
         if not mi.is_null('series') and mi.series.strip():
@@ -292,7 +349,37 @@ class MetadataSingleDialogBase(ResizableDialog):
             self.comments.current_val = mi.comments
 
     def fetch_metadata(self, *args):
-        pass # TODO: fetch metadata
+        d = FullFetch(self.cover.pixmap(), self)
+        ret = d.start(title=self.title.current_val, authors=self.authors.current_val,
+                identifiers=self.identifiers.current_val)
+        if ret == d.Accepted:
+            from calibre.ebooks.metadata.sources.base import msprefs
+            mi = d.book
+            dummy = Metadata(_('Unknown'))
+            for f in msprefs['ignore_fields']:
+                if ':' not in f:
+                    setattr(mi, f, getattr(dummy, f))
+            if mi is not None:
+                self.update_from_mi(mi)
+            if d.cover_pixmap is not None:
+                self.cover.current_val = pixmap_to_data(d.cover_pixmap)
+
+    def configure_metadata(self):
+        from calibre.gui2.preferences import show_config_widget
+        gui = self.parent()
+        show_config_widget('Sharing', 'Metadata download', parent=self,
+                gui=gui, never_shutdown=True)
+
+    def download_cover(self, *args):
+        from calibre.gui2.metadata.single_download import CoverFetch
+        d = CoverFetch(self.cover.pixmap(), self)
+        ret = d.start(self.title.current_val, self.authors.current_val,
+                self.identifiers.current_val)
+        if ret == d.Accepted:
+            if d.cover_pixmap is not None:
+                self.cover.current_val = pixmap_to_data(d.cover_pixmap)
+
+
     # }}}
 
     def apply_changes(self):
@@ -303,7 +390,7 @@ class MetadataSingleDialogBase(ResizableDialog):
                     return False
                 self.books_to_refresh |= getattr(widget, 'books_to_refresh',
                         set([]))
-            except IOError, err:
+            except IOError as err:
                 if err.errno == 13: # Permission denied
                     import traceback
                     fname = err.filename if err.filename else 'file'
@@ -337,11 +424,13 @@ class MetadataSingleDialogBase(ResizableDialog):
         gprefs['metasingle_window_geometry3'] = bytearray(self.saveGeometry())
 
     # Dialog use methods {{{
-    def start(self, row_list, current_row, view_slot=None):
+    def start(self, row_list, current_row, view_slot=None,
+            set_current_callback=None):
         self.row_list = row_list
         self.current_row = current_row
         if view_slot is not None:
             self.view_format.connect(view_slot)
+        self.set_current_callback = set_current_callback
         self.do_one(apply_changes=False)
         ret = self.exec_()
         self.break_cycles()
@@ -373,6 +462,7 @@ class MetadataSingleDialogBase(ResizableDialog):
     def break_cycles(self):
         # Break any reference cycles that could prevent python
         # from garbage collecting this dialog
+        self.set_current_callback = self.db = None
         def disconnect(signal):
             try:
                 signal.disconnect()
@@ -385,6 +475,14 @@ class MetadataSingleDialogBase(ResizableDialog):
                 disconnect(x.clicked)
     # }}}
 
+class Splitter(QSplitter):
+
+    frame_resized = pyqtSignal(object)
+
+    def resizeEvent(self, ev):
+        self.frame_resized.emit(ev)
+        return QSplitter.resizeEvent(self, ev)
+
 class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
 
     def do_layout(self):
@@ -408,7 +506,8 @@ class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
 
         sto = QWidget.setTabOrder
         sto(self.button_box, self.fetch_metadata_button)
-        sto(self.fetch_metadata_button, self.title)
+        sto(self.fetch_metadata_button, self.config_metadata_button)
+        sto(self.config_metadata_button, self.title)
 
         def create_row(row, one, two, three, col=1, icon='forward.png'):
             ql = BuddyLabel(one)
@@ -425,7 +524,8 @@ class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
             sto(one, two)
             sto(two, three)
 
-        tl.addWidget(self.swap_title_author_button, 0, 0, 2, 1)
+        tl.addWidget(self.swap_title_author_button, 0, 0, 1, 1)
+        tl.addWidget(self.manage_authors_button, 1, 0, 1, 1)
 
         create_row(0, self.title, self.deduce_title_sort_button, self.title_sort)
         sto(self.title_sort, self.authors)
@@ -434,16 +534,18 @@ class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
         create_row(2, self.series, self.remove_unused_series_button,
                 self.series_index, icon='trash.png')
         sto(self.series_index, self.swap_title_author_button)
+        sto(self.swap_title_author_button, self.manage_authors_button)
 
         tl.addWidget(self.formats_manager, 0, 6, 3, 1)
 
-        self.splitter = QSplitter(Qt.Horizontal, self)
+        self.splitter = Splitter(Qt.Horizontal, self)
         self.splitter.addWidget(self.cover)
+        self.splitter.frame_resized.connect(self.cover.frame_resized)
         l.addWidget(self.splitter)
         self.tabs[0].gb = gb = QGroupBox(_('Change cover'), self)
         gb.l = l = QGridLayout()
         gb.setLayout(l)
-        sto(self.swap_title_author_button, self.cover.buttons[0])
+        sto(self.manage_authors_button, self.cover.buttons[0])
         for i, b in enumerate(self.cover.buttons[:3]):
             l.addWidget(b, 0, i, 1, 1)
             sto(b, self.cover.buttons[i+1])
@@ -457,10 +559,16 @@ class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
         w.setLayout(w.l)
         l.setMargin(0)
         self.splitter.addWidget(w)
-        def create_row2(row, widget, button=None):
+        def create_row2(row, widget, button=None, front_button=None):
             row += 1
             ql = BuddyLabel(widget)
-            l.addWidget(ql, row, 0, 1, 1)
+            if front_button:
+                ltl = QHBoxLayout()
+                ltl.addWidget(front_button)
+                ltl.addWidget(ql)
+                l.addLayout(ltl, row, 0, 1, 1)
+            else:
+                l.addWidget(ql, row, 0, 1, 1)
             l.addWidget(widget, row, 1, 1, 2 if button is None else 1)
             if button is not None:
                 l.addWidget(button, row, 2, 1, 1)
@@ -475,9 +583,11 @@ class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
         create_row2(1, self.rating)
         sto(self.rating, self.tags)
         create_row2(2, self.tags, self.tags_editor_button)
-        sto(self.tags_editor_button, self.isbn)
-        create_row2(3, self.isbn)
-        sto(self.isbn, self.timestamp)
+        sto(self.tags_editor_button, self.paste_isbn_button)
+        sto(self.paste_isbn_button, self.identifiers)
+        create_row2(3, self.identifiers, self.clear_identifiers_button,
+                                        front_button=self.paste_isbn_button)
+        sto(self.clear_identifiers_button, self.timestamp)
         create_row2(4, self.timestamp, self.timestamp.clear_button)
         sto(self.timestamp.clear_button, self.pubdate)
         create_row2(5, self.pubdate, self.pubdate.clear_button)
@@ -486,7 +596,8 @@ class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
         self.tabs[0].spc_two = QSpacerItem(10, 10, QSizePolicy.Expanding,
                 QSizePolicy.Expanding)
         l.addItem(self.tabs[0].spc_two, 8, 0, 1, 3)
-        l.addWidget(self.fetch_metadata_button, 9, 0, 1, 3)
+        l.addWidget(self.fetch_metadata_button, 9, 0, 1, 2)
+        l.addWidget(self.config_metadata_button, 9, 2, 1, 1)
 
         self.tabs[0].gb2 = gb = QGroupBox(_('Co&mments'), self)
         gb.l = l = QVBoxLayout()
@@ -498,10 +609,27 @@ class MetadataSingleDialog(MetadataSingleDialogBase): # {{{
 
 # }}}
 
-class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
+class DragTrackingWidget(QWidget): # {{{
+
+    def __init__(self, parent, on_drag_enter):
+        QWidget.__init__(self, parent)
+        self.on_drag_enter = on_drag_enter
+
+    def dragEnterEvent(self, ev):
+        self.on_drag_enter.emit()
+
+# }}}
+
+class MetadataSingleDialogAlt1(MetadataSingleDialogBase): # {{{
 
     cc_two_column = False
     one_line_comments_toolbar = True
+    use_toolbutton_for_config_metadata = False
+
+    on_drag_enter = pyqtSignal()
+
+    def handle_drag_enter(self):
+        self.central_widget.setCurrentIndex(1)
 
     def do_layout(self):
         self.central_widget.clear()
@@ -509,7 +637,8 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
         self.labels = []
         sto = QWidget.setTabOrder
 
-        self.tabs.append(QWidget(self))
+        self.on_drag_enter.connect(self.handle_drag_enter)
+        self.tabs.append(DragTrackingWidget(self, self.on_drag_enter))
         self.central_widget.addTab(self.tabs[0], _("&Metadata"))
         self.tabs[0].l = QGridLayout()
         self.tabs[0].setLayout(self.tabs[0].l)
@@ -519,6 +648,10 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
         self.tabs[1].l = QGridLayout()
         self.tabs[1].setLayout(self.tabs[1].l)
 
+        # accept drop events so we can automatically switch to the second tab to
+        # drop covers and formats
+        self.tabs[0].setAcceptDrops(True)
+
         # Tab 0
         tab0 = self.tabs[0]
 
@@ -527,7 +660,11 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
         self.tabs[0].l.addWidget(gb, 0, 0, 1, 1)
         gb.setLayout(tl)
 
-        sto(self.button_box, self.title)
+        self.button_box_layout.insertWidget(1, self.fetch_metadata_button)
+        self.button_box_layout.insertWidget(2, self.config_metadata_button)
+        sto(self.button_box, self.fetch_metadata_button)
+        sto(self.fetch_metadata_button, self.config_metadata_button)
+        sto(self.config_metadata_button, self.title)
 
         def create_row(row, widget, tab_to, button=None, icon=None, span=1):
             ql = BuddyLabel(widget)
@@ -545,6 +682,8 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
                     sto(widget, tab_to)
 
         tl.addWidget(self.swap_title_author_button, 0, 0, 2, 1)
+        tl.addWidget(self.manage_authors_button, 2, 0, 1, 1)
+        tl.addWidget(self.paste_isbn_button, 11, 0, 1, 1)
 
         create_row(0, self.title, self.title_sort,
                    button=self.deduce_title_sort_button, span=2,
@@ -562,9 +701,13 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
         create_row(8, self.pubdate, self.publisher,
                    button=self.pubdate.clear_button, icon='trash.png')
         create_row(9, self.publisher, self.timestamp)
-        create_row(10, self.timestamp, self.isbn,
+        create_row(10, self.timestamp, self.identifiers,
                    button=self.timestamp.clear_button, icon='trash.png')
-        create_row(11, self.isbn, self.comments)
+        create_row(11, self.identifiers, self.comments,
+                   button=self.clear_identifiers_button, icon='trash.png')
+        sto(self.clear_identifiers_button, self.swap_title_author_button)
+        sto(self.swap_title_author_button, self.manage_authors_button)
+        sto(self.manage_authors_button, self.paste_isbn_button)
         tl.addItem(QSpacerItem(1, 1, QSizePolicy.Fixed, QSizePolicy.Expanding),
                    12, 1, 1 ,1)
 
@@ -580,7 +723,7 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
             sr.setWidget(w)
             gbl.addWidget(sr)
             self.tabs[0].l.addWidget(gb, 0, 1, 1, 1)
-            sto(self.isbn, gb)
+            sto(self.identifiers, gb)
 
         w = QGroupBox(_('&Comments'), tab0)
         sp = QSizePolicy()
@@ -604,7 +747,6 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
         gb = QGroupBox(_('Change cover'), tab1)
         l = QGridLayout()
         gb.setLayout(l)
-        sto(self.swap_title_author_button, self.cover.buttons[0])
         for i, b in enumerate(self.cover.buttons[:3]):
             l.addWidget(b, 0, i, 1, 1)
             sto(b, self.cover.buttons[i+1])
@@ -616,7 +758,6 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
         wgl.addWidget(gb)
         wgl.addItem(QSpacerItem(10, 10, QSizePolicy.Expanding,
             QSizePolicy.Expanding))
-        wgl.addWidget(self.fetch_metadata_button)
         wgl.addItem(QSpacerItem(10, 10, QSizePolicy.Expanding,
             QSizePolicy.Expanding))
         wgl.addWidget(self.formats_manager)
@@ -631,10 +772,148 @@ class MetadataSingleDialogAlt(MetadataSingleDialogBase): # {{{
 
 # }}}
 
+class MetadataSingleDialogAlt2(MetadataSingleDialogBase): # {{{
 
-def edit_metadata(db, row_list, current_row, parent=None, view_slot=None):
-    d = MetadataSingleDialog(db, parent)
-    d.start(row_list, current_row, view_slot=view_slot)
+    cc_two_column = False
+    one_line_comments_toolbar = True
+    use_toolbutton_for_config_metadata = False
+
+    def do_layout(self):
+        self.central_widget.clear()
+        self.labels = []
+        sto = QWidget.setTabOrder
+
+        self.central_widget.tabBar().setVisible(False)
+        tab0 = QWidget(self)
+        self.central_widget.addTab(tab0, _("&Metadata"))
+        l = QGridLayout()
+        tab0.setLayout(l)
+
+        # Basic metadata in col 0
+        tl = QGridLayout()
+        gb = QGroupBox(_('Basic metadata'), tab0)
+        l.addWidget(gb, 0, 0, 1, 1)
+        gb.setLayout(tl)
+
+        self.button_box_layout.insertWidget(1, self.fetch_metadata_button)
+        self.button_box_layout.insertWidget(2, self.config_metadata_button)
+        sto(self.button_box, self.fetch_metadata_button)
+        sto(self.fetch_metadata_button, self.config_metadata_button)
+        sto(self.config_metadata_button, self.title)
+
+        def create_row(row, widget, tab_to, button=None, icon=None, span=1):
+            ql = BuddyLabel(widget)
+            tl.addWidget(ql, row, 1, 1, 1)
+            tl.addWidget(widget, row, 2, 1, 1)
+            if button is not None:
+                tl.addWidget(button, row, 3, span, 1)
+                if icon is not None:
+                    button.setIcon(QIcon(I(icon)))
+            if tab_to is not None:
+                if button is not None:
+                    sto(widget, button)
+                    sto(button, tab_to)
+                else:
+                    sto(widget, tab_to)
+
+        tl.addWidget(self.swap_title_author_button, 0, 0, 2, 1)
+        tl.addWidget(self.manage_authors_button, 2, 0, 2, 1)
+        tl.addWidget(self.paste_isbn_button, 11, 0, 1, 1)
+
+        create_row(0, self.title, self.title_sort,
+                   button=self.deduce_title_sort_button, span=2,
+                   icon='auto_author_sort.png')
+        create_row(1, self.title_sort, self.authors)
+        create_row(2, self.authors, self.author_sort,
+                   button=self.deduce_author_sort_button,
+                   span=2, icon='auto_author_sort.png')
+        create_row(3, self.author_sort, self.series)
+        create_row(4, self.series, self.series_index,
+                   button=self.remove_unused_series_button, icon='trash.png')
+        create_row(5, self.series_index, self.tags)
+        create_row(6, self.tags, self.rating, button=self.tags_editor_button)
+        create_row(7, self.rating, self.pubdate)
+        create_row(8, self.pubdate, self.publisher,
+                   button=self.pubdate.clear_button, icon='trash.png')
+        create_row(9, self.publisher, self.timestamp)
+        create_row(10, self.timestamp, self.identifiers,
+                   button=self.timestamp.clear_button, icon='trash.png')
+        create_row(11, self.identifiers, self.comments,
+                   button=self.clear_identifiers_button, icon='trash.png')
+        sto(self.clear_identifiers_button, self.swap_title_author_button)
+        sto(self.swap_title_author_button, self.manage_authors_button)
+        sto(self.manage_authors_button, self.paste_isbn_button)
+        tl.addItem(QSpacerItem(1, 1, QSizePolicy.Fixed, QSizePolicy.Expanding),
+                   12, 1, 1 ,1)
+
+        # Custom metadata in col 1
+        w = getattr(self, 'custom_metadata_widgets_parent', None)
+        if w is not None:
+            gb = QGroupBox(_('Custom metadata'), tab0)
+            gbl = QVBoxLayout()
+            gb.setLayout(gbl)
+            sr = QScrollArea(gb)
+            sr.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+            sr.setWidgetResizable(True)
+            sr.setBackgroundRole(QPalette.Base)
+            sr.setFrameStyle(QFrame.NoFrame)
+            sr.setWidget(w)
+            gbl.addWidget(sr)
+            l.addWidget(gb, 0, 1, 1, 1)
+            sp = QSizePolicy()
+            sp.setVerticalStretch(10)
+            sp.setHorizontalPolicy(QSizePolicy.Minimum)
+            sp.setVerticalPolicy(QSizePolicy.Expanding)
+            gb.setSizePolicy(sp)
+            self.set_custom_metadata_tab_order()
+
+        # comments span col 0 & 1
+        w = QGroupBox(_('Comments'), tab0)
+        sp = QSizePolicy()
+        sp.setVerticalStretch(10)
+        sp.setHorizontalPolicy(QSizePolicy.Expanding)
+        sp.setVerticalPolicy(QSizePolicy.Expanding)
+        w.setSizePolicy(sp)
+        lb = QHBoxLayout()
+        w.setLayout(lb)
+        lb.addWidget(self.comments)
+        l.addWidget(w, 1, 0, 1, 2)
+
+        # Cover & formats in col 3
+        gb = QGroupBox(_('Cover'), tab0)
+        lb = QGridLayout()
+        gb.setLayout(lb)
+        lb.addWidget(self.cover, 0, 0, 1, 3, alignment=Qt.AlignCenter)
+        sto(self.manage_authors_button, self.cover.buttons[0])
+        for i, b in enumerate(self.cover.buttons[:3]):
+            lb.addWidget(b, 1, i, 1, 1)
+            sto(b, self.cover.buttons[i+1])
+        hl = QHBoxLayout()
+        for b in self.cover.buttons[3:]:
+            hl.addWidget(b)
+        sto(self.cover.buttons[-2], self.cover.buttons[-1])
+        lb.addLayout(hl, 2, 0, 1, 3)
+        l.addWidget(gb, 0, 2, 1, 1)
+        l.addWidget(self.formats_manager, 1, 2, 1, 1)
+        sto(self.cover.buttons[-1], self.formats_manager)
+
+        self.formats_manager.formats.setMaximumWidth(10000)
+        self.formats_manager.formats.setIconSize(QSize(32, 32))
+
+# }}}
+
+
+editors = {'default': MetadataSingleDialog, 'alt1': MetadataSingleDialogAlt1,
+           'alt2': MetadataSingleDialogAlt2}
+
+def edit_metadata(db, row_list, current_row, parent=None, view_slot=None,
+        set_current_callback=None):
+    cls = gprefs.get('edit_metadata_single_layout', '')
+    if cls not in editors:
+        cls = 'default'
+    d = editors[cls](db, parent)
+    d.start(row_list, current_row, view_slot=view_slot,
+            set_current_callback=set_current_callback)
     return d.changed, d.rows_to_refresh
 
 if __name__ == '__main__':
