@@ -15,7 +15,7 @@ from calibre.utils.config import tweaks, prefs
 from calibre.utils.date import parse_date, now, UNDEFINED_DATE
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.utils.pyparsing import ParseException
-from calibre.ebooks.metadata import title_sort
+from calibre.ebooks.metadata import title_sort, author_to_author_sort
 from calibre.ebooks.metadata.opf2 import metadata_to_opf
 from calibre import prints
 
@@ -191,7 +191,8 @@ class CacheRow(list): # {{{
             if is_comp:
                 id = list.__getitem__(self, 0)
                 self._must_do = False
-                mi = self.db.get_metadata(id, index_is_id=True)
+                mi = self.db.get_metadata(id, index_is_id=True,
+                                          get_user_categories=False)
                 for c in self._composites:
                     self[c] =  mi.get(self._composites[c])
         return list.__getitem__(self, col)
@@ -390,58 +391,66 @@ class ResultCache(SearchQueryParser): # {{{
     def build_numeric_relop_dict(self):
         self.numeric_search_relops = {
                         '=':[1, lambda r, q: r == q],
-                        '>':[1, lambda r, q: r > q],
-                        '<':[1, lambda r, q: r < q],
+                        '>':[1, lambda r, q: r is not None and r > q],
+                        '<':[1, lambda r, q: r is not None and r < q],
                         '!=':[2, lambda r, q: r != q],
-                        '>=':[2, lambda r, q: r >= q],
-                        '<=':[2, lambda r, q: r <= q]
+                        '>=':[2, lambda r, q: r is not None and r >= q],
+                        '<=':[2, lambda r, q: r is not None and r <= q]
                     }
 
     def get_numeric_matches(self, location, query, candidates, val_func = None):
         matches = set([])
         if len(query) == 0:
             return matches
-        if query == 'false':
-            query = '0'
-        elif query == 'true':
-            query = '!=0'
-        relop = None
-        for k in self.numeric_search_relops.keys():
-            if query.startswith(k):
-                (p, relop) = self.numeric_search_relops[k]
-                query = query[p:]
-        if relop is None:
-                (p, relop) = self.numeric_search_relops['=']
 
         if val_func is None:
             loc = self.field_metadata[location]['rec_index']
             val_func = lambda item, loc=loc: item[loc]
-
+        q = ''
+        cast = adjust = lambda x: x
         dt = self.field_metadata[location]['datatype']
-        if dt == 'int':
-            cast = (lambda x: int (x))
-            adjust = lambda x: x
-        elif  dt == 'rating':
-            cast = (lambda x: int (x))
-            adjust = lambda x: x/2
-        elif dt in ('float', 'composite'):
-            cast = lambda x : float (x)
-            adjust = lambda x: x
-        else: # count operation
-            cast = (lambda x: int (x))
-            adjust = lambda x: x
 
-        if len(query) > 1:
-            mult = query[-1:].lower()
-            mult = {'k':1024.,'m': 1024.**2, 'g': 1024.**3}.get(mult, 1.0)
-            if mult != 1.0:
-                query = query[:-1]
+        if query == 'false':
+            if dt == 'rating' or location == 'cover':
+                relop = lambda x,y: not bool(x)
+            else:
+                relop = lambda x,y: x is None
+        elif query == 'true':
+            if dt == 'rating' or location == 'cover':
+                relop = lambda x,y: bool(x)
+            else:
+                relop = lambda x,y: x is not None
         else:
-            mult = 1.0
-        try:
-            q = cast(query) * mult
-        except:
-            return matches
+            relop = None
+            for k in self.numeric_search_relops.keys():
+                if query.startswith(k):
+                    (p, relop) = self.numeric_search_relops[k]
+                    query = query[p:]
+            if relop is None:
+                    (p, relop) = self.numeric_search_relops['=']
+
+            if dt == 'int':
+                cast = lambda x: int (x)
+            elif  dt == 'rating':
+                cast = lambda x: 0 if x is None else int (x)
+                adjust = lambda x: x/2
+            elif dt in ('float', 'composite'):
+                cast = lambda x : float (x)
+            else: # count operation
+                cast = (lambda x: int (x))
+
+            if len(query) > 1:
+                mult = query[-1:].lower()
+                mult = {'k':1024.,'m': 1024.**2, 'g': 1024.**3}.get(mult, 1.0)
+                if mult != 1.0:
+                    query = query[:-1]
+            else:
+                mult = 1.0
+            try:
+                q = cast(query) * mult
+            except:
+                raise ParseException(query, len(query),
+                                     'Non-numeric value in query', self)
 
         for id_ in candidates:
             item = self._data[id_]
@@ -450,10 +459,8 @@ class ResultCache(SearchQueryParser): # {{{
             try:
                 v = cast(val_func(item))
             except:
-                v = 0
-            if not v:
-                v = 0
-            else:
+                v = None
+            if v:
                 v = adjust(v)
             if relop(v, q):
                 matches.add(item[0])
@@ -547,10 +554,14 @@ class ResultCache(SearchQueryParser): # {{{
         return matchkind, query
 
     def get_bool_matches(self, location, query, candidates):
-        bools_are_tristate = tweaks['bool_custom_columns_are_tristate'] != 'no'
+        bools_are_tristate = self.db_prefs.get('bools_are_tristate')
         loc = self.field_metadata[location]['rec_index']
         matches = set()
         query = icu_lower(query)
+        if query not in (_('no'), _('unchecked'), '_no', 'false',
+                         _('yes'), _('checked'), '_yes', 'true',
+                         _('empty'), _('blank'), '_empty'):
+            raise ParseException(_('Invalid boolean query "{0}"').format(query))
         for id_ in candidates:
             item = self._data[id_]
             if item is None:
@@ -621,8 +632,11 @@ class ResultCache(SearchQueryParser): # {{{
                         terms.add(l)
                 if terms:
                     for l in terms:
-                        matches |= self.get_matches(l, query,
-                            candidates=candidates, allow_recursion=allow_recursion)
+                        try:
+                            matches |= self.get_matches(l, query,
+                                candidates=candidates, allow_recursion=allow_recursion)
+                        except:
+                            pass
                     return matches
 
             if location in self.field_metadata:
@@ -698,7 +712,10 @@ class ResultCache(SearchQueryParser): # {{{
             for loc in location: # location is now an array of field indices
                 if loc == db_col['authors']:
                     ### DB stores authors with commas changed to bars, so change query
-                    q = query.replace(',', '|');
+                    if matchkind == REGEXP_MATCH:
+                        q = query.replace(',', r'\|');
+                    else:
+                        q = query.replace(',', '|');
                 else:
                     q = query
 
@@ -743,7 +760,7 @@ class ResultCache(SearchQueryParser): # {{{
 
                     if loc not in exclude_fields: # time for text matching
                         if is_multiple_cols[loc] is not None:
-                            vals = item[loc].split(is_multiple_cols[loc])
+                            vals = [v.strip() for v in item[loc].split(is_multiple_cols[loc])]
                         else:
                             vals = [item[loc]] ### make into list to make _match happy
                         if _match(q, vals, matchkind):
@@ -766,7 +783,7 @@ class ResultCache(SearchQueryParser): # {{{
         else:
             q = query
             if search_restriction:
-                q = u'%s (%s)' % (search_restriction, query)
+                q = u'(%s) and (%s)' % (search_restriction, query)
         if not q:
             if set_restriction_count:
                 self.search_restriction_book_count = len(self._map)
@@ -947,7 +964,7 @@ class ResultCache(SearchQueryParser): # {{{
         if not fields:
             fields = [('timestamp', False)]
 
-        keyg = SortKeyGenerator(fields, self.field_metadata, self._data)
+        keyg = SortKeyGenerator(fields, self.field_metadata, self._data, self.db_prefs)
         self._map.sort(key=keyg)
 
         tmap = list(itertools.repeat(False, len(self._data)))
@@ -970,9 +987,10 @@ class SortKey(object):
 
 class SortKeyGenerator(object):
 
-    def __init__(self, fields, field_metadata, data):
+    def __init__(self, fields, field_metadata, data, db_prefs):
         from calibre.utils.icu import sort_key
         self.field_metadata = field_metadata
+        self.db_prefs = db_prefs
         self.orders = [1 if x[1] else -1 for x in fields]
         self.entries = [(x[0], field_metadata[x[0]]) for x in fields]
         self.library_order = tweaks['title_series_sorting'] == 'library_order'
@@ -992,9 +1010,9 @@ class SortKeyGenerator(object):
                 if sb == 'date':
                     try:
                         val = parse_date(val)
-                        dt = 'datetime'
                     except:
-                        pass
+                        val = UNDEFINED_DATE
+                    dt = 'datetime'
                 elif sb == 'number':
                     try:
                         val = float(val)
@@ -1023,12 +1041,16 @@ class SortKeyGenerator(object):
                 if val:
                     sep = fm['is_multiple']
                     if sep:
-                        val = sep.join(sorted(val.split(sep),
+                        if fm['display'].get('is_names', False):
+                            val = sep.join(
+                                [author_to_author_sort(v) for v in val.split(sep)])
+                        else:
+                            val = sep.join(sorted(val.split(sep),
                                               key=self.string_sort_key))
                 val = self.string_sort_key(val)
 
             elif dt == 'bool':
-                if tweaks['bool_custom_columns_are_tristate'] == 'no':
+                if not self.db_prefs.get('bools_are_tristate'):
                     val = {True: 1, False: 2, None: 2}.get(val, 2)
                 else:
                     val = {True: 1, False: 2, None: 3}.get(val, 3)
