@@ -4,8 +4,8 @@ Created on 25 May 2010
 @author: charles
 '''
 import copy, traceback
+from collections import OrderedDict
 
-from calibre.utils.ordered_dict import OrderedDict
 from calibre.utils.config import tweaks
 
 class TagsIcons(dict):
@@ -35,7 +35,7 @@ category_icon_map = {
                     'custom:'    : 'column.png',
                     'user:'      : 'tb_folder.png',
                     'search'     : 'search.png',
-                    'identifiers': 'id_card.png'
+                    'identifiers': 'identifiers.png'
             }
 
 
@@ -188,7 +188,7 @@ class FieldMetadata(dict):
                             'datatype':'text',
                            'is_multiple':None,
                            'kind':'field',
-                           'name':None,
+                           'name':_('Author Sort'),
                            'search_terms':['author_sort'],
                            'is_custom':False,
                            'is_category':False,
@@ -238,7 +238,7 @@ class FieldMetadata(dict):
                            'datatype':'datetime',
                            'is_multiple':None,
                            'kind':'field',
-                           'name':_('Date'),
+                           'name':_('Modified'),
                            'search_terms':['last_modified'],
                            'is_custom':False,
                            'is_category':False,
@@ -258,7 +258,7 @@ class FieldMetadata(dict):
                            'datatype':'text',
                            'is_multiple':None,
                            'kind':'field',
-                           'name':None,
+                           'name':_('Path'),
                            'search_terms':[],
                            'is_custom':False,
                            'is_category':False,
@@ -308,7 +308,7 @@ class FieldMetadata(dict):
                            'datatype':'float',
                            'is_multiple':None,
                            'kind':'field',
-                           'name':_('Size (MB)'),
+                           'name':_('Size'),
                            'search_terms':['size'],
                            'is_custom':False,
                            'is_category':False,
@@ -364,11 +364,12 @@ class FieldMetadata(dict):
             self._tb_cats[k]['display'] = {}
             self._tb_cats[k]['is_editable'] = True
             self._add_search_terms_to_map(k, v['search_terms'])
-        for x in ('timestamp', 'last_modified'):
-            self._tb_cats[x]['display'] = {
+        self._tb_cats['timestamp']['display'] = {
                         'date_format': tweaks['gui_timestamp_display_format']}
         self._tb_cats['pubdate']['display'] = {
                         'date_format': tweaks['gui_pubdate_display_format']}
+        self._tb_cats['last_modified']['display'] = {
+                        'date_format': tweaks['gui_last_modified_display_format']}
         self.custom_field_prefix = '#'
         self.get = self._tb_cats.get
 
@@ -399,6 +400,13 @@ class FieldMetadata(dict):
                 if self._tb_cats[k]['kind']=='field' and
                    self._tb_cats[k]['datatype'] is not None]
 
+    def displayable_field_keys(self):
+        return [k for k in self._tb_cats.keys()
+                if self._tb_cats[k]['kind']=='field' and
+                   self._tb_cats[k]['datatype'] is not None and
+                   k not in ('au_map', 'marked', 'ondevice', 'cover') and
+                   not self.is_series_index(k)]
+
     def standard_field_keys(self):
         return [k for k in self._tb_cats.keys()
                 if self._tb_cats[k]['kind']=='field' and
@@ -442,6 +450,11 @@ class FieldMetadata(dict):
     def is_custom_field(self, key):
         return key.startswith(self.custom_field_prefix)
 
+    def is_series_index(self, key):
+        m = self[key]
+        return (m['datatype'] == 'float' and key.endswith('_index') and
+                key[:-6] in self)
+
     def key_to_label(self, key):
         if 'label' not in self._tb_cats[key]:
             return key
