@@ -504,7 +504,8 @@ class BuiltinFormat_date(BuiltinFormatterFunction):
             'MMM  : the abbreviated localized month name (e.g. "Jan" to "Dec"). '
             'MMMM : the long localized month name (e.g. "January" to "December"). '
             'yy   : the year as two digit number (00 to 99). '
-            'yyyy : the year as four digit number.')
+            'yyyy : the year as four digit number. '
+            'iso  : the date with time and timezone. Must be the only format present')
 
     def evaluate(self, formatter, kwargs, mi, locals, val, format_string):
         if not val:
@@ -548,8 +549,22 @@ class BuiltinCapitalize(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals, val):
         return capitalize(val)
 
+class BuiltinBooksize(BuiltinFormatterFunction):
+    name = 'booksize'
+    arg_count = 0
+    doc = _('booksize() -- return value of the field capitalized')
+
+    def evaluate(self, formatter, kwargs, mi, locals):
+        if mi.book_size is not None:
+            try:
+                return str(mi.book_size)
+            except:
+                pass
+        return ''
+
 builtin_add         = BuiltinAdd()
 builtin_assign      = BuiltinAssign()
+builtin_booksize    = BuiltinBooksize()
 builtin_capitalize  = BuiltinCapitalize()
 builtin_cmp         = BuiltinCmp()
 builtin_contains    = BuiltinContains()
