@@ -14,7 +14,7 @@ from contextlib import nested, closing
 
 
 from calibre import browser, __appname__, iswindows, \
-                    strftime, preferred_encoding
+                    strftime, preferred_encoding, as_unicode
 from calibre.ebooks.BeautifulSoup import BeautifulSoup, NavigableString, CData, Tag
 from calibre.ebooks.metadata.opf2 import OPFCreator
 from calibre import entity_to_unicode
@@ -71,7 +71,8 @@ class BasicNewsRecipe(Recipe):
     #: Number of levels of links to follow on article webpages
     recursions             = 0
 
-    #: Delay between consecutive downloads in seconds
+    #: Delay between consecutive downloads in seconds. The argument may be a
+    #: floating point number to indicate a more precise time.
     delay                  = 0
 
     #: Publication type
@@ -700,10 +701,17 @@ class BasicNewsRecipe(Recipe):
         for attr in self.remove_attributes:
             for x in soup.findAll(attrs={attr:True}):
                 del x[attr]
-        for base in list(soup.findAll(['base', 'iframe'])):
+        for base in list(soup.findAll(['base', 'iframe', 'canvas', 'embed',
+            'command', 'datalist', 'video', 'audio'])):
             base.extract()
 
         ans = self.postprocess_html(soup, first_fetch)
+
+        # Nuke HTML5 tags
+        for x in ans.findAll(['article', 'aside', 'header', 'footer', 'nav',
+            'figcaption', 'figure', 'section']):
+            x.name = 'div'
+
         if job_info:
             url, f, a, feed_len = job_info
             try:
@@ -832,7 +840,13 @@ class BasicNewsRecipe(Recipe):
         fetcher.image_url_processor = self.image_url_processor
         res, path, failures = fetcher.start_fetch(url), fetcher.downloaded_paths, fetcher.failed_links
         if not res or not os.path.exists(res):
-            raise Exception(_('Could not fetch article. Run with -vv to see the reason'))
+            msg = _('Could not fetch article.') + ' '
+            if self.debug:
+                msg += _('The debug traceback is available earlier in this log')
+            else:
+                msg += _('Run with -vv to see the reason')
+            raise Exception(msg)
+
         return res, path, failures
 
     def fetch_article(self, url, dir, f, a, num_of_feeds):
@@ -895,9 +909,6 @@ class BasicNewsRecipe(Recipe):
             feeds = feeds[:2]
         self.has_single_feed = len(feeds) == 1
 
-        if self.use_embedded_content is None:
-            self.use_embedded_content = feeds[0].has_embedded_content()
-
         index = os.path.join(self.output_dir, 'index.html')
 
         html = self.feeds2index(feeds)
@@ -932,7 +943,9 @@ class BasicNewsRecipe(Recipe):
                     url = None
                 if not url:
                     continue
-                func, arg = (self.fetch_embedded_article, article) if self.use_embedded_content else \
+                func, arg = (self.fetch_embedded_article, article) \
+                            if self.use_embedded_content or (self.use_embedded_content == None and feed.has_embedded_content()) \
+                            else \
                             ((self.fetch_obfuscated_article if self.articles_are_obfuscated \
                               else self.fetch_article), url)
                 req = WorkRequest(func, (arg, art_dir, f, a, len(feed)),
@@ -974,8 +987,8 @@ class BasicNewsRecipe(Recipe):
         self.cover_path = None
         try:
             cu = self.get_cover_url()
-        except Exception, err:
-            self.log.error(_('Could not download cover: %s')%str(err))
+        except Exception as err:
+            self.log.error(_('Could not download cover: %s')%as_unicode(err))
             self.log.debug(traceback.format_exc())
         else:
             if not cu:
@@ -1306,11 +1319,11 @@ class BasicNewsRecipe(Recipe):
                                           oldest_article=self.oldest_article,
                                           max_articles_per_feed=self.max_articles_per_feed,
                                           get_article_url=self.get_article_url))
-            except Exception, err:
+            except Exception as err:
                 feed = Feed()
                 msg = 'Failed feed: %s'%(title if title else url)
                 feed.populate_from_preparsed_feed(msg, [])
-                feed.description = repr(err)
+                feed.description = as_unicode(err)
                 parsed_feeds.append(feed)
                 self.log.exception(msg)
 
@@ -1456,7 +1469,7 @@ class CalibrePeriodical(BasicNewsRecipe):
                 'http://news.calibre-ebook.com/subscribed_files/%s/0/temp.downloaded_recipe'
                 % self.calibre_periodicals_slug
                     ).read()
-        except Exception, e:
+        except Exception as e:
             if hasattr(e, 'getcode') and e.getcode() == 403:
                 raise DownloadDenied(
                         _('You do not have permission to download this issue.'
