@@ -6,7 +6,7 @@ __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, calendar
+import os, calendar, zipfile
 from threading import RLock
 from datetime import timedelta
 
@@ -24,16 +24,15 @@ def iterate_over_builtin_recipe_files():
     exclude = ['craigslist', 'iht', 'outlook_india', 'toronto_sun',
             'indian_express', 'india_today', 'livemint']
     d = os.path.dirname
-    base = os.path.join(d(d(d(d(d(d(os.path.abspath(__file__))))))), 'resources', 'recipes')
-    for x in os.walk(base):
-        for f in x[-1]:
-            fbase, ext = os.path.splitext(f)
-            if ext != '.recipe' or fbase in exclude:
-                continue
-            f = os.path.join(x[0], f)
-            rid = os.path.splitext(os.path.relpath(f, base).replace(os.sep,
-                '/'))[0]
-            yield rid, os.path.join(x[0], f)
+    base = os.path.join(d(d(d(d(d(d(os.path.abspath(__file__))))))), 'recipes')
+    for f in os.listdir(base):
+        fbase, ext = os.path.splitext(f)
+        if ext != '.recipe' or fbase in exclude:
+            continue
+        f = os.path.join(base, f)
+        rid = os.path.splitext(os.path.relpath(f, base).replace(os.sep,
+            '/'))[0]
+        yield rid, f
 
 
 def serialize_recipe(urn, recipe_class):
@@ -178,6 +177,10 @@ def download_builtin_recipe(urn):
     br = browser()
     return br.open_novisit('http://status.calibre-ebook.com/recipe/'+urn).read()
 
+def get_builtin_recipe(urn):
+    with zipfile.ZipFile(P('builtin_recipes.zip'), 'r') as zf:
+        return zf.read(urn+'.recipe')
+
 def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
     for x in get_builtin_recipe_collection():
         if x.get('title') == title:
@@ -194,7 +197,7 @@ def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
                     else:
                         log.exception(
                         'Failed to download recipe, using builtin version')
-            return P('recipes/%s.recipe'%urn, data=True)
+            return get_builtin_recipe(urn)
 
 def get_builtin_recipe_by_id(id_, log=None, download_recipe=False):
     for x in get_builtin_recipe_collection():
@@ -212,7 +215,7 @@ def get_builtin_recipe_by_id(id_, log=None, download_recipe=False):
                     else:
                         log.exception(
                         'Failed to download recipe, using builtin version')
-            return P('recipes/%s.recipe'%urn, data=True)
+            return get_builtin_recipe(urn)
 
 class SchedulerConfig(object):
 
