@@ -6,10 +6,10 @@ __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, copy
+import copy, zipfile
 
 from PyQt4.Qt import QAbstractItemModel, QVariant, Qt, QColor, QFont, QIcon, \
-        QModelIndex, pyqtSignal
+        QModelIndex, pyqtSignal, QPixmap
 
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.gui2 import NONE
@@ -17,7 +17,8 @@ from calibre.utils.localization import get_language
 from calibre.web.feeds.recipes.collection import \
         get_builtin_recipe_collection, get_custom_recipe_collection, \
         SchedulerConfig, download_builtin_recipe, update_custom_recipe, \
-        add_custom_recipe, remove_custom_recipe, get_custom_recipe
+        add_custom_recipe, remove_custom_recipe, get_custom_recipe, \
+        get_builtin_recipe
 from calibre.utils.pyparsing import ParseException
 
 class NewsTreeItem(object):
@@ -93,12 +94,13 @@ class NewsCategory(NewsTreeItem):
 
 class NewsItem(NewsTreeItem):
 
-    def __init__(self, urn, title, default_icon, custom_icon,
+    def __init__(self, urn, title, default_icon, custom_icon, favicons, zf,
             builtin, custom, scheduler_config, parent):
         NewsTreeItem.__init__(self, builtin, custom, scheduler_config, parent)
         self.urn, self.title = urn, title
         self.icon = self.default_icon = None
         self.default_icon = default_icon
+        self.favicons, self.zf = favicons, zf
         if 'custom:' in self.urn:
             self.icon = custom_icon
 
@@ -107,9 +109,16 @@ class NewsItem(NewsTreeItem):
             return QVariant(self.title)
         if role == Qt.DecorationRole:
             if self.icon is None:
-                icon = I('news/%s.png'%self.urn[8:])
-                if os.path.exists(icon):
-                    self.icon = QVariant(QIcon(icon))
+                icon = '%s.png'%self.urn[8:]
+                p = QPixmap()
+                if icon in self.favicons:
+                    try:
+                        with zipfile.ZipFile(self.zf, 'r') as zf:
+                            p.loadFromData(zf.read(self.favicons[icon]))
+                    except:
+                        pass
+                if not p.isNull():
+                    self.icon = QVariant(QIcon(p))
                 else:
                     self.icon = self.default_icon
             return self.icon
@@ -130,6 +139,12 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
         self.custom_icon = QVariant(QIcon(I('user_profile.png')))
         self.builtin_recipe_collection = get_builtin_recipe_collection()
         self.scheduler_config = SchedulerConfig()
+        try:
+            with zipfile.ZipFile(P('builtin_recipes.zip'), 'r') as zf:
+                self.favicons = dict([(x.filename, x) for x in zf.infolist() if
+                    x.filename.endswith('.png')])
+        except:
+            self.favicons = {}
         self.do_refresh()
 
     def get_builtin_recipe(self, urn, download=True):
@@ -139,7 +154,7 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
             except:
                 import traceback
                 traceback.print_exc()
-        return P('recipes/%s.recipe'%urn, data=True)
+        return get_builtin_recipe(urn)
 
     def get_recipe(self, urn, download=True):
         coll = self.custom_recipe_collection if urn.startswith('custom:') \
@@ -166,11 +181,13 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
 
     def do_refresh(self, restrict_to_urns=set([])):
         self.custom_recipe_collection = get_custom_recipe_collection()
+        zf = P('builtin_recipes.zip')
 
         def factory(cls, parent, *args):
             args = list(args)
             if cls is NewsItem:
-                args.extend([self.default_icon, self.custom_icon])
+                args.extend([self.default_icon, self.custom_icon,
+                    self.favicons, zf])
             args += [self.builtin_recipe_collection,
                      self.custom_recipe_collection, self.scheduler_config,
                      parent]
